# SPDX-License-Identifier: Apache-2.0
import unittest

from onnx import defs, AttributeProto


class TestSchema(unittest.TestCase):

    def test_get_schema(self) -> None:
        defs.get_schema("Relu")

    def test_typecheck(self) -> None:
        defs.get_schema("Conv")

    def test_attr_default_value(self) -> None:
        v = defs.get_schema(
            "BatchNormalization").attributes['epsilon'].default_value
        self.assertEqual(type(v), AttributeProto)
        self.assertEqual(v.type, AttributeProto.FLOAT)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
import tempfile
import unittest
import uuid

import numpy as np  # type: ignore
import shutil

import os
import os.path as Path

import onnx
from onnx import checker, helper, shape_inference
from onnx import ModelProto, TensorProto
from onnx.external_data_helper import set_external_data
from onnx.external_data_helper import convert_model_to_external_data
from onnx.external_data_helper import convert_model_from_external_data
from onnx.external_data_helper import load_external_data_for_model, load_external_data_for_tensor
from onnx.numpy_helper import to_array, from_array
from typing import Any, Tuple, List
import pytest  # type: ignore
import sys


class TestLoadExternalDataBase(unittest.TestCase):

    def setUp(self) -> None:
        self.temp_dir: str = tempfile.mkdtemp()
        self.initializer_value = np.arange(6).reshape(3, 2).astype(np.float32) + 512
        self.attribute_value = np.arange(6).reshape(2, 3).astype(np.float32) + 256
        self.model_filename = self.create_test_model()

    def tearDown(self) -> None:
        shutil.rmtree(self.temp_dir)

    def get_temp_model_filename(self) -> str:
        return os.path.join(self.temp_dir, str(uuid.uuid4()) + '.onnx')

    def create_external_data_tensor(self, value: List[Any], tensor_name: str) -> TensorProto:
        tensor = from_array(np.array(value))
        tensor.name = tensor_name
        tensor_filename = f"{tensor_name}.bin"
        set_external_data(tensor, location=tensor_filename)

        with open(os.path.join(self.temp_dir, tensor_filename), 'wb') as data_file:
            data_file.write(tensor.raw_data)
        tensor.ClearField('raw_data')
        tensor.data_location = onnx.TensorProto.EXTERNAL
        return tensor

    def create_test_model(self) -> str:

        constant_node = onnx.helper.make_node(
            'Constant',
            inputs=[],
            outputs=['values'],
            value=self.create_external_data_tensor(self.attribute_value, "attribute_value")
        )

        initializers = [self.create_external_data_tensor(self.initializer_value, "input_value")]
        inputs = [helper.make_tensor_value_info("input_value",
                                                onnx.TensorProto.FLOAT,
                                                self.initializer_value.shape)]

        graph = helper.make_graph([constant_node], "test_graph",
                                  inputs=inputs, outputs=[],
                                  initializer=initializers)
        model = helper.make_model(graph)

        model_filename = os.path.join(self.temp_dir, "model.onnx")
        with open(model_filename, "wb") as model_file:
            model_file.write(model.SerializeToString())

        return model_filename

    def test_check_model(self) -> None:
        checker.check_model(self.model_filename)


class TestLoadExternalData(TestLoadExternalDataBase):

    def test_load_external_data(self) -> None:
        model = onnx.load_model(self.model_filename)
        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))

    def test_load_external_data_for_model(self) -> None:
        model = onnx.load_model(self.model_filename, load_external_data=False)
        load_external_data_for_model(model, self.temp_dir)
        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))

    def test_save_external_data(self) -> None:
        model = onnx.load_model(self.model_filename)

        temp_dir = os.path.join(self.temp_dir, "save_copy")
        os.mkdir(temp_dir)
        new_model_filename = os.path.join(temp_dir, 'model.onnx')
        onnx.save_model(model, new_model_filename)

        new_model = onnx.load_model(new_model_filename)
        initializer_tensor = new_model.graph.initializer[0]
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = new_model.graph.node[0].attribute[0].t
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))


class TestLoadExternalDataSingleFile(TestLoadExternalDataBase):

    def create_external_data_tensors(self, tensors_data: List[Tuple[List[Any], Any]]) -> List[TensorProto]:
        tensor_filename = "tensors.bin"
        tensors = []

        with open(os.path.join(self.temp_dir, tensor_filename), 'ab') as data_file:
            for (value, tensor_name) in tensors_data:
                tensor = from_array(np.array(value))
                offset = data_file.tell()
                if offset % 4096 != 0:
                    data_file.write(b"\0" * (4096 - offset % 4096))
                    offset = offset + 4096 - offset % 4096

                data_file.write(tensor.raw_data)
                set_external_data(tensor, location=tensor_filename, offset=offset, length=data_file.tell() - offset)
                tensor.name = tensor_name
                tensor.ClearField("raw_data")
                tensor.data_location = onnx.TensorProto.EXTERNAL
                tensors.append(tensor)

        return tensors

    def test_load_external_single_file_data(self) -> None:
        model = onnx.load_model(self.model_filename)

        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))

    def test_save_external_single_file_data(self) -> None:
        model = onnx.load_model(self.model_filename)

        temp_dir = os.path.join(self.temp_dir, "save_copy")
        os.mkdir(temp_dir)
        new_model_filename = os.path.join(temp_dir, 'model.onnx')
        onnx.save_model(model, new_model_filename)

        new_model = onnx.load_model(new_model_filename)
        initializer_tensor = new_model.graph.initializer[0]
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = new_model.graph.node[0].attribute[0].t
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))


class TestSaveAllTensorsAsExternalData(TestLoadExternalDataBase):

    def setUp(self) -> None:
        self.temp_dir: str = tempfile.mkdtemp()
        self.initializer_value = np.arange(6).reshape(3, 2).astype(np.float32) + 512
        self.attribute_value = np.arange(6).reshape(2, 3).astype(np.float32) + 256
        self.model = self.create_test_model_proto()

    def create_data_tensors(self, tensors_data: List[Tuple[List[Any], Any]]) -> List[TensorProto]:
        tensors = []
        for (value, tensor_name) in tensors_data:
            tensor = from_array(np.array(value))
            tensor.name = tensor_name
            tensors.append(tensor)

        return tensors

    def create_test_model_proto(self) -> ModelProto:
        tensors = self.create_data_tensors([
            (self.attribute_value, "attribute_value"),
            (self.initializer_value, "input_value"),
        ])

        constant_node = onnx.helper.make_node(
            'Constant',
            inputs=[],
            outputs=['values'],
            value=tensors[0]
        )

        inputs = [helper.make_tensor_value_info("input_value",
                                                onnx.TensorProto.FLOAT,
                                                self.initializer_value.shape)]

        graph = helper.make_graph([constant_node], "test_graph",
                                  inputs=inputs, outputs=[],
                                  initializer=[tensors[1]])
        return helper.make_model(graph)

    def test_check_model(self) -> None:
        checker.check_model(self.model)

    def test_convert_model_to_external_data_with_size_threshold(self) -> None:
        model_file_path = self.get_temp_model_filename()

        convert_model_to_external_data(self.model, size_threshold=1024)
        onnx.save_model(self.model, model_file_path)

        model = onnx.load_model(model_file_path)
        initializer_tensor = model.graph.initializer[0]
        self.assertFalse(initializer_tensor.HasField("data_location"))

    def test_convert_model_to_external_data_without_size_threshold(self) -> None:
        model_file_path = self.get_temp_model_filename()
        convert_model_to_external_data(self.model, size_threshold=0)
        onnx.save_model(self.model, model_file_path)

        model = onnx.load_model(model_file_path)
        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(initializer_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

    def test_convert_model_to_external_data_from_one_file_with_location(self) -> None:
        model_file_path = self.get_temp_model_filename()
        external_data_file = str(uuid.uuid4())

        convert_model_to_external_data(self.model, size_threshold=0, all_tensors_to_one_file=True, location=external_data_file)
        onnx.save_model(self.model, model_file_path)

        self.assertTrue(Path.isfile(os.path.join(self.temp_dir, external_data_file)))

        model = onnx.load_model(model_file_path)

        # test convert model from external data
        convert_model_from_external_data(model)
        model_file_path = self.get_temp_model_filename()
        onnx.save_model(model, model_file_path)
        model = onnx.load_model(model_file_path)
        initializer_tensor = model.graph.initializer[0]
        self.assertFalse(len(initializer_tensor.external_data))
        self.assertEqual(initializer_tensor.data_location, TensorProto.DEFAULT)
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertFalse(len(attribute_tensor.external_data))
        self.assertEqual(attribute_tensor.data_location, TensorProto.DEFAULT)
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))

    def test_convert_model_to_external_data_from_one_file_without_location_uses_model_name(self) -> None:
        model_file_path = self.get_temp_model_filename()

        convert_model_to_external_data(self.model, size_threshold=0, all_tensors_to_one_file=True)
        onnx.save_model(self.model, model_file_path)

        self.assertTrue(Path.isfile(model_file_path))
        self.assertTrue(Path.isfile(os.path.join(self.temp_dir, model_file_path)))

    def test_convert_model_to_external_data_one_file_per_tensor_without_attribute(self) -> None:
        model_file_path = self.get_temp_model_filename()

        convert_model_to_external_data(self.model, size_threshold=0, all_tensors_to_one_file=False, convert_attribute=False)
        onnx.save_model(self.model, model_file_path)

        self.assertTrue(Path.isfile(model_file_path))
        self.assertTrue(Path.isfile(os.path.join(self.temp_dir, "input_value")))
        self.assertFalse(Path.isfile(os.path.join(self.temp_dir, "attribute_value")))

    def test_convert_model_to_external_data_one_file_per_tensor_with_attribute(self) -> None:
        model_file_path = self.get_temp_model_filename()

        convert_model_to_external_data(self.model, size_threshold=0, all_tensors_to_one_file=False, convert_attribute=True)
        onnx.save_model(self.model, model_file_path)

        self.assertTrue(Path.isfile(model_file_path))
        self.assertTrue(Path.isfile(os.path.join(self.temp_dir, "input_value")))
        self.assertTrue(Path.isfile(os.path.join(self.temp_dir, "attribute_value")))

    def test_convert_model_to_external_data_does_not_convert_attribute_values(self) -> None:
        model_file_path = self.get_temp_model_filename()

        convert_model_to_external_data(self.model, size_threshold=0, convert_attribute=False, all_tensors_to_one_file=False)
        onnx.save_model(self.model, model_file_path)

        self.assertTrue(Path.isfile(os.path.join(self.temp_dir, "input_value")))
        self.assertFalse(Path.isfile(os.path.join(self.temp_dir, "attribute_value")))

        model = onnx.load_model(model_file_path)
        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(initializer_tensor.HasField("data_location"))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertFalse(attribute_tensor.HasField("data_location"))

    def test_convert_model_to_external_data_converts_attribute_values(self) -> None:
        model_file_path = self.get_temp_model_filename()

        convert_model_to_external_data(self.model, size_threshold=0, convert_attribute=True)
        onnx.save_model(self.model, model_file_path)

        model = onnx.load_model(model_file_path)

        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))
        self.assertTrue(initializer_tensor.HasField("data_location"))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))
        self.assertTrue(attribute_tensor.HasField("data_location"))

    def test_save_model_does_not_convert_to_external_data_and_saves_the_model(self) -> None:
        model_file_path = self.get_temp_model_filename()
        onnx.save_model(self.model, model_file_path, save_as_external_data=False)
        self.assertTrue(Path.isfile(model_file_path))

        model = onnx.load_model(model_file_path)
        initializer_tensor = model.graph.initializer[0]
        self.assertFalse(initializer_tensor.HasField("data_location"))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertFalse(attribute_tensor.HasField("data_location"))

    def test_save_model_does_convert_and_saves_the_model(self) -> None:
        model_file_path = self.get_temp_model_filename()
        onnx.save_model(self.model,
                        model_file_path,
                        save_as_external_data=True,
                        all_tensors_to_one_file=True,
                        location=None,
                        size_threshold=0,
                        convert_attribute=False)

        model = onnx.load_model(model_file_path)

        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(initializer_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertFalse(attribute_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))

    def test_save_model_without_loading_external_data(self) -> None:
        model_file_path = self.get_temp_model_filename()
        onnx.save_model(self.model,
                        model_file_path,
                        save_as_external_data=True,
                        location=None,
                        size_threshold=0,
                        convert_attribute=False)
        # Save without load_external_data
        model = onnx.load_model(model_file_path, load_external_data=False)
        onnx.save_model(model,
                        model_file_path,
                        save_as_external_data=True,
                        location=None,
                        size_threshold=0,
                        convert_attribute=False)
        # Load the saved model again; Only works if the saved path is under the same directory
        model = onnx.load_model(model_file_path)

        initializer_tensor = model.graph.initializer[0]
        self.assertTrue(initializer_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(initializer_tensor), self.initializer_value))

        attribute_tensor = model.graph.node[0].attribute[0].t
        self.assertFalse(attribute_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(attribute_tensor), self.attribute_value))

    def test_save_model_with_existing_raw_data_should_override(self) -> None:
        model_file_path = self.get_temp_model_filename()
        original_raw_data = self.model.graph.initializer[0].raw_data
        onnx.save_model(self.model, model_file_path, save_as_external_data=True, size_threshold=0)
        self.assertTrue(Path.isfile(model_file_path))

        model = onnx.load_model(model_file_path, load_external_data=False)
        initializer_tensor = model.graph.initializer[0]
        initializer_tensor.raw_data = b'dummpy_raw_data'
        # If raw_data and external tensor exist at the same time, override existing raw_data
        load_external_data_for_tensor(initializer_tensor, self.temp_dir)
        self.assertEqual(initializer_tensor.raw_data, original_raw_data)


class TestExternalDataToArray(unittest.TestCase):
    def setUp(self) -> None:
        self.temp_dir: str = tempfile.mkdtemp()
        self.model_file_path: str = os.path.join(self.temp_dir, 'model.onnx')
        self.large_data = np.random.rand(10, 60, 100).astype(np.float32)
        self.small_data = (200, 300)
        self.model = self.create_test_model()

    def tearDown(self) -> None:
        shutil.rmtree(self.temp_dir)

    def get_temp_model_filename(self) -> str:
        return os.path.join(self.temp_dir, str(uuid.uuid4()) + '.onnx')

    def create_test_model(self) -> ModelProto:
        X = helper.make_tensor_value_info('X', TensorProto.FLOAT, self.large_data.shape)
        input_init = helper.make_tensor(name='X', data_type=TensorProto.FLOAT,
            dims=self.large_data.shape, vals=self.large_data.tobytes(), raw=True)

        shape_data = np.array(self.small_data, np.int64)
        shape_init = helper.make_tensor(name='Shape', data_type=TensorProto.INT64,
            dims=shape_data.shape, vals=shape_data.tobytes(), raw=True)
        C = helper.make_tensor_value_info('C', TensorProto.INT64, self.small_data)

        reshape = onnx.helper.make_node(
            'Reshape',
            inputs=['X', 'Shape'],
            outputs=['Y'],
        )
        cast = onnx.helper.make_node(
            'Cast',
            inputs=['Y'],
            outputs=['C'],
            to=getattr(TensorProto, 'INT64')
        )

        graph_def = helper.make_graph(
            [reshape, cast],
            'test-model',
            [X],
            [C],
            initializer=[input_init, shape_init],
        )
        model = helper.make_model(graph_def, producer_name='onnx-example')
        return model

    def test_check_model(self) -> None:
        checker.check_model(self.model)

    def test_reshape_inference_with_external_data_fail(self) -> None:
        onnx.save_model(self.model, self.model_file_path, save_as_external_data=True, all_tensors_to_one_file=False, size_threshold=0)
        model_without_external_data = onnx.load(self.model_file_path, load_external_data=False)
        # Shape inference of Reshape uses ParseData
        # ParseData cannot handle external data and should throw the error as follows:
        # Cannot parse data from external tensors. Please load external data into raw data for tensor: Shape
        self.assertRaises(shape_inference.InferenceError, shape_inference.infer_shapes,
            model_without_external_data, strict_mode=True)

    def test_to_array_with_external_data(self) -> None:
        onnx.save_model(self.model,
                        self.model_file_path,
                        save_as_external_data=True,
                        all_tensors_to_one_file=False,
                        size_threshold=0)
        # raw_data of external tensor is not loaded
        model = onnx.load(self.model_file_path, load_external_data=False)
        # Specify self.temp_dir to load external tensor
        loaded_large_data = to_array(model.graph.initializer[0], self.temp_dir)
        self.assertTrue(np.allclose(loaded_large_data, self.large_data))

    def test_save_model_with_external_data_multiple_times(self) -> None:
        # Test onnx.save should respectively handle typical tensor and external tensor properly
        # 1st save: save two tensors which have raw_data
        # Only w_large will be stored as external tensors since it's larger than 1024
        onnx.save_model(self.model,
                        self.model_file_path,
                        save_as_external_data=True,
                        all_tensors_to_one_file=False,
                        location=None,
                        size_threshold=1024,
                        convert_attribute=True)
        model_without_loading_external = onnx.load(self.model_file_path, load_external_data=False)
        large_input_tensor = model_without_loading_external.graph.initializer[0]
        self.assertTrue(large_input_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(large_input_tensor, self.temp_dir), self.large_data))

        small_shape_tensor = model_without_loading_external.graph.initializer[1]
        self.assertTrue(not small_shape_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(small_shape_tensor), self.small_data))

        # 2nd save: one tensor has raw_data (small); one external tensor (large)
        # Save them both as external tensors this time
        onnx.save_model(model_without_loading_external,
                        self.model_file_path,
                        save_as_external_data=True,
                        all_tensors_to_one_file=False,
                        location=None,
                        size_threshold=0,
                        convert_attribute=True)

        model_without_loading_external = onnx.load(self.model_file_path, load_external_data=False)
        large_input_tensor = model_without_loading_external.graph.initializer[0]
        self.assertTrue(large_input_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(large_input_tensor, self.temp_dir), self.large_data))

        small_shape_tensor = model_without_loading_external.graph.initializer[1]
        self.assertTrue(small_shape_tensor.HasField("data_location"))
        self.assertTrue(np.allclose(to_array(small_shape_tensor, self.temp_dir), self.small_data))


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
import onnx
import unittest
from onnx import helper, parser, GraphProto


class TestBasicFunctions(unittest.TestCase):
    def check_graph(self, graph: GraphProto) -> None:
        self.assertTrue(len(graph.node) == 3)
        self.assertTrue(graph.node[0].op_type == "MatMul")
        self.assertTrue(graph.node[1].op_type == "Add")
        self.assertTrue(graph.node[2].op_type == "Softmax")

    def test_parse_graph(self) -> None:
        input = '''
           agraph (float[N, 128] X, float[128,10] W, float[10] B) => (float[N] C)
           {
              T = MatMul(X, W)
              S = Add(T, B)
              C = Softmax(S)
           }
           '''
        graph = onnx.parser.parse_graph(input)
        self.check_graph(graph)

    def test_parse_model(self) -> None:
        input = '''
           <
             ir_version: 7,
             opset_import: [ "" : 10, "com.microsoft": 1]
           >
           agraph (float[N, 128] X, float[128,10] W, float[10] B) => (float[N] C)
           {
              T = MatMul(X, W)
              S = Add(T, B)
              C = Softmax(S)
           }
           '''
        model = onnx.parser.parse_model(input)
        self.assertTrue(model.ir_version == 7)
        self.assertTrue(len(model.opset_import) == 2)
        self.check_graph(model.graph)

    def test_parse_graph_error(self) -> None:
        input = '''
           agraph (float[N, 128] X, float[128,10] W, float[10] B) => (float[N] C)
           {
              T = MatMul[X, W]
              S = Add(T, B)
              C = Softmax(S)
           }
           '''
        self.assertRaises(onnx.parser.ParseError,
                          lambda: onnx.parser.parse_graph(input))

    def test_parse_model_error(self) -> None:
        input = '''
           <
             ir_version: 7,
             opset_import: [ "" : 10   "com.microsoft": 1]
           >
           agraph (float[N, 128] X, float[128,10] W, float[10] B) => (float[N] C)
           {
              T = MatMul(X, W)
              S = Add(T, B)
              C = Softmax(S)
           }
           '''
        self.assertRaises(onnx.parser.ParseError,
                          lambda: onnx.parser.parse_model(input))


if __name__ == '__main__':
    unittest.main()



class InferenceError(Exception):
    ...

def infer_shapes(b: bytes, check_type: bool, strict_mode: bool, data_prop: bool) -> bytes: ...

def infer_shapes_path(model_path: str, output_path: str, check_type: bool, strict_mode: bool, data_prop: bool) -> None: ...



from distutils.spawn import find_executable
from distutils import sysconfig, log
import setuptools
import setuptools.command.build_py
import setuptools.command.develop
import setuptools.command.build_ext

from collections import namedtuple
from contextlib import contextmanager
import glob
import os
import shlex
import subprocess
import sys
import platform
from textwrap import dedent
import multiprocessing


TOP_DIR = os.path.realpath(os.path.dirname(__file__))
SRC_DIR = os.path.join(TOP_DIR, 'onnxsim')
CMAKE_BUILD_DIR = os.path.join(TOP_DIR, '.setuptools-cmake-build')

WINDOWS = (os.name == 'nt')

CMAKE = find_executable('cmake')

install_requires = []
setup_requires = []

USE_MSVC_STATIC_RUNTIME = bool(os.getenv('USE_MSVC_STATIC_RUNTIME', '0') == '1')
ONNX_ML = not bool(os.getenv('ONNX_ML') == '0')
ONNX_VERIFY_PROTO3 = bool(os.getenv('ONNX_VERIFY_PROTO3') == '1')
ONNX_NAMESPACE = os.getenv('ONNX_NAMESPACE', 'onnx')
ONNX_BUILD_TESTS = bool(os.getenv('ONNX_BUILD_TESTS') == '1')
ONNX_OPT_USE_SYSTEM_PROTOBUF = bool(os.getenv('ONNX_OPT_USE_SYSTEM_PROTOBUF', '0') == '1')

DEBUG = bool(os.getenv('DEBUG'))
COVERAGE = bool(os.getenv('COVERAGE'))

try:
    version = subprocess.check_output(['git', 'describe', '--tags', '--abbrev=0'],
                                      cwd=TOP_DIR).decode('ascii').strip()
    if version[0] == 'v':
        version = version[1:]
except (OSError, subprocess.CalledProcessError):
    version = "unknown"

try:
    git_version = subprocess.check_output(['git', 'rev-parse', 'HEAD'],
                                          cwd=TOP_DIR).decode('ascii').strip()
except (OSError, subprocess.CalledProcessError):
    git_version = None

if os.getenv("ONNXSIM_SDIST") is not None:
    version = '0.4.10'
    git_version = None

VersionInfo = namedtuple('VersionInfo', ['version', 'git_version'])(
    version=version,
    git_version=git_version
)

assert CMAKE, 'Could not find "cmake" executable!'

@contextmanager
def cd(path):
    if not os.path.isabs(path):
        raise RuntimeError('Can only cd to absolute path, got: {}'.format(path))
    orig_path = os.getcwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(orig_path)


class ONNXCommand(setuptools.Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass


class create_version(ONNXCommand):
    def run(self):
        with open(os.path.join(SRC_DIR, 'version.py'), 'w') as f:
            f.write(dedent('''\
            # This file is generated by setup.py. DO NOT EDIT!

            version = '{version}'
            git_version = '{git_version}'
            '''.format(**dict(VersionInfo._asdict()))))


class cmake_build(setuptools.Command):
    """
    Compiles everything when `python setupmnm.py build` is run using cmake.

    Custom args can be passed to cmake by specifying the `CMAKE_ARGS`
    environment variable.

    The number of CPUs used by `make` can be specified by passing `-j<ncpus>`
    to `setup.py build`.  By default all CPUs are used.
    """
    user_options = [
        (str('jobs='), str('j'), str('Specifies the number of jobs to use with make'))
    ]

    built = False

    def initialize_options(self):
        self.jobs = None

    def finalize_options(self):
        self.set_undefined_options('build', ('parallel', 'jobs'))
        if self.jobs is None and os.getenv("MAX_JOBS") is not None:
            self.jobs = os.getenv("MAX_JOBS")
        self.jobs = multiprocessing.cpu_count() if self.jobs is None else int(self.jobs)

    def run(self):
        if cmake_build.built:
            return
        cmake_build.built = True
        if not os.path.exists(CMAKE_BUILD_DIR):
            os.makedirs(CMAKE_BUILD_DIR)

        with cd(CMAKE_BUILD_DIR):
            build_type = 'Release'
            # configure
            cmake_args = [
                CMAKE,
                '-DPython_INCLUDE_DIR={}'.format(sysconfig.get_python_inc()),
                '-DPython_EXECUTABLE={}'.format(sys.executable),
                # For pybind11
                '-DPYTHON_EXECUTABLE={}'.format(sys.executable),
                '-DBUILD_ONNX_PYTHON=OFF',
                '-DONNXSIM_PYTHON=ON',
                '-DONNXSIM_BUILTIN_ORT=OFF',
                '-DONNX_USE_LITE_PROTO=OFF',
                '-DCMAKE_EXPORT_COMPILE_COMMANDS=ON',
                '-DONNX_NAMESPACE={}'.format(ONNX_NAMESPACE),
                '-DPY_EXT_SUFFIX={}'.format(
                    sysconfig.get_config_var('EXT_SUFFIX') or ''),
                '-DONNX_OPT_USE_SYSTEM_PROTOBUF={}'.format(
                    'ON' if ONNX_OPT_USE_SYSTEM_PROTOBUF else 'OFF'),
            ]
            if COVERAGE:
                cmake_args.append('-DONNX_COVERAGE=ON')
            if COVERAGE or DEBUG:
                # in order to get accurate coverage information, the
                # build needs to turn off optimizations
                build_type = 'Debug'
            cmake_args.append('-DCMAKE_BUILD_TYPE=%s' % build_type)
            if WINDOWS:
                cmake_args.extend([
                    # we need to link with libpython on windows, so
                    # passing python version to window in order to
                    # find python in cmake
                    '-DPY_VERSION={}'.format('{0}.{1}'.format(* \
                                                              sys.version_info[:2])),
                ])
                if USE_MSVC_STATIC_RUNTIME:
                    cmake_args.append('-DONNX_USE_MSVC_STATIC_RUNTIME=ON')
                if platform.architecture()[0] == '64bit':
                    cmake_args.extend(['-A', 'x64', '-T', 'host=x64'])
                else:
                    cmake_args.extend(['-A', 'Win32', '-T', 'host=x86'])
            if ONNX_ML:
                cmake_args.append('-DONNX_ML=1')
            if ONNX_VERIFY_PROTO3:
                cmake_args.append('-DONNX_VERIFY_PROTO3=1')
            if ONNX_BUILD_TESTS:
                cmake_args.append('-DONNX_BUILD_TESTS=ON')
            if 'CMAKE_ARGS' in os.environ:
                extra_cmake_args = shlex.split(os.environ['CMAKE_ARGS'])
                # prevent crossfire with downstream scripts
                del os.environ['CMAKE_ARGS']
                log.info('Extra cmake args: {}'.format(extra_cmake_args))
                cmake_args.extend(extra_cmake_args)
            cmake_args.append(TOP_DIR)
            print(f"Run command {cmake_args}")
            subprocess.check_call(cmake_args)

            build_args = [CMAKE, '--build', os.curdir, '--target onnxsim_cpp2py_export']
            if WINDOWS:
                build_args.extend(['--config', build_type])
                build_args.extend(['--', '/maxcpucount:{}'.format(self.jobs)])
            else:
                build_args.extend(['--', '-j', str(self.jobs)])
            print(f"Run command {build_args}")
            subprocess.check_call(build_args)


class build_py(setuptools.command.build_py.build_py):
    def run(self):
        self.run_command('create_version')
        return setuptools.command.build_py.build_py.run(self)


class develop(setuptools.command.develop.develop):
    def run(self):
        self.run_command('build_py')
        setuptools.command.develop.develop.run(self)


class build_ext(setuptools.command.build_ext.build_ext):
    def run(self):
        self.run_command('cmake_build')
        setuptools.command.build_ext.build_ext.run(self)

    def build_extensions(self):
        for ext in self.extensions:
            fullname = self.get_ext_fullname(ext.name)
            filename = os.path.basename(self.get_ext_filename(fullname))

            lib_path = CMAKE_BUILD_DIR
            if os.name == 'nt':
                debug_lib_dir = os.path.join(lib_path, "Debug")
                release_lib_dir = os.path.join(lib_path, "Release")
                if os.path.exists(debug_lib_dir):
                    lib_path = debug_lib_dir
                elif os.path.exists(release_lib_dir):
                    lib_path = release_lib_dir
            src = os.path.join(lib_path, filename)
            dst_dir = os.path.join(os.path.realpath(
                self.build_lib), "onnxsim")
            dst = os.path.join(dst_dir, filename)
            os.makedirs(dst_dir, exist_ok=True)
            self.copy_file(src, dst)


cmdclass = {
    'create_version': create_version,
    'cmake_build': cmake_build,
    'build_ext': build_ext,
    'build_py': build_py,
    'develop': develop,
}

ext_modules = [
    setuptools.Extension(
        name=str('onnxsim.onnxsim_cpp2py_export'),
        sources=[])
]

# no need to do fancy stuff so far
packages = setuptools.find_packages()

# Though we depend on onnxruntime, it has three different packages:
# onnxruntime, onnxruntime-gpu and onnxruntime-noopenmp.
# The solution is, we publish two packages, a wheel named onnxsim-no-ort
# and a sdist package named onnxsim, onnxsim depends on onnxsim-no-ort,
# and also check if one of onnxruntime packages is installed, and depends
# on onnxruntime when no existing installed packages.
install_requires.extend([
    'onnx',
    'rich',
])

setup_requires.append('pytest-runner')

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setuptools.setup(
    name=os.getenv("ONNXSIM_PKG_NAME", "onnxsim"),
    version=VersionInfo.version,
    description='Simplify your ONNX model',
    ext_modules=ext_modules,
    cmdclass=cmdclass,
    packages=packages,
    license='Apache License v2.0',
    include_package_data=True,
    install_requires=install_requires,
    setup_requires=setup_requires,
    author='ONNX Simplifier Authors',
    author_email='daquexian566@gmail.com',
    url='https://github.com/daquexian/onnx-simplifier',
    keywords='deep-learning ONNX',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development'
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'onnxsim=onnxsim:main',
        ],
    },
)



from collections import defaultdict
from typing import Callable, Any, Optional

import onnx
from rich.table import Table
from rich.text import Text
from rich import print


__all__ = ['ModelInfo', 'print_simplifying_info']


def human_readable_size(num, suffix="B"):
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return f"{num:3.1f}{unit}{suffix}"
        num /= 1024.0
    return f"{num:.1f}Yi{suffix}"


class ModelInfo:
    """
    Model info contains:
    1. Num of every op
    2. Model size
    TODO: 
    Based on onnx runtime, get
    1、FLOPs
    2、forward memory footprint
    3、memory access
    4、compute density
    """

    def __init__(self, model: onnx.ModelProto):
        self.op_nums = defaultdict(int)
        # TODO: include ops in subgraph
        for node in model.graph.node:
            self.op_nums[node.op_type] += 1
        self.model_size = model.ByteSize()


def print_simplifying_info(model_ori: onnx.ModelProto, model_opt: onnx.ModelProto) -> None:
    """
    --------------------------------------------------------
    |             | original model | simplified model |
    --------------------------------------------------------
    | ****        | ****           | ****             |
    --------------------------------------------------------
    | Model Size  | ****           | ****             |
    --------------------------------------------------------
    """
    ori_info = ModelInfo(model_ori)
    opt_info = ModelInfo(model_opt)
    table = Table()
    table.add_column('')
    table.add_column('Original Model')
    table.add_column('Simplified Model')

    def add_row(table: Table, key, ori_data, opt_data, is_better: Callable[[Any, Any], Any], postprocess: Optional[Callable[[Any], Any]] = None) -> None:
        if postprocess is None:
            postprocess = str
        if is_better(opt_data, ori_data):
            table.add_row(key, postprocess(ori_data), Text(
                postprocess(opt_data), style='bold green1'))
        else:
            table.add_row(key, postprocess(ori_data), postprocess(opt_data))

    for key in sorted(list(set(ori_info.op_nums.keys()) | set(opt_info.op_nums.keys()))):
        add_row(table, key, ori_info.op_nums[key],
                opt_info.op_nums[key], lambda opt, ori: opt < ori)
    add_row(
        table, 'Model Size', ori_info.model_size, opt_info.model_size, lambda opt, ori: opt < ori, postprocess=human_readable_size)
    print(table)



# SPDX-License-Identifier: Apache-2.0
import onnx
from onnx import helper, TensorProto, shape_inference, version_converter, ValueInfoProto
from typing import List, Dict, Any, Union, Callable, Optional, cast
import string
import numpy as np  # type: ignore
import unittest

import time

#####################################################################################
# Every test creates a model containing a single operator from the lowest possible
# opset version, upgrades it to the most recent opset version and then runs checker +
# shape inference on the upgraded model.
####################################################################################

latest_opset = onnx.defs.onnx_opset_version()
tested_ops = []


class TestAutomaticUpgrade(unittest.TestCase):

    def _test_op_upgrade(
        self,
        op: str,
        from_opset: int,
        input_shapes: List[Union[List[Optional[int]], str]] = [[3, 4, 5]],
        output_shapes: List[List[Optional[int]]] = [[3, 4, 5]],
        input_types: Union[List[Any], None] = None,
        output_types: Union[List[Any], None] = None,
        initializer: List[Any] = [],
        attrs: Dict[str, Any] = {},
        seq_inputs: List[int] = [],
        seq_outputs: List[int] = [],
        optional_inputs: List[int] = [],
        optional_outputs: List[int] = []
    ) -> None:
        global tested_ops
        tested_ops.append(op)

        n_inputs = len(input_shapes)
        letters = list(string.ascii_lowercase)[:n_inputs]
        input_names = [
            letter if shape != '' else '' for (letter, shape) in zip(letters, input_shapes)
        ]
        if input_types is None:
            input_types = [TensorProto.FLOAT] * n_inputs
        is_sequence = [0 if id not in seq_inputs else 1 for id in range(n_inputs)]
        is_optional = [0 if id not in optional_inputs else 1 for id in range(n_inputs)]
        # turn empty strings into [0] to ease type analysis, even though those entries
        # will be ignored
        input_shapes_cast = cast(List[List[int]],
                [[0] if isinstance(shape, str) else shape for shape in input_shapes]
        )
        inputs: List[ValueInfoProto] = []
        for (name, ttype, shape, is_seq, is_opt) in \
                zip(input_names, input_types, input_shapes_cast, is_sequence, is_optional):
            if name != '':
                if is_seq:
                    inputs += [helper.make_tensor_sequence_value_info(name, ttype, shape)]
                elif is_opt:
                    type_proto = helper.make_tensor_type_proto(ttype, shape)
                    optional_type_proto = helper.make_optional_type_proto(type_proto)
                    inputs += [helper.make_value_info(name, optional_type_proto)]
                else:
                    inputs += [helper.make_tensor_value_info(name, ttype, shape)]

        n_outputs = len(output_shapes)
        output_names = list(string.ascii_lowercase)[n_inputs:n_inputs + n_outputs]
        if output_types is None:
            output_types = [TensorProto.FLOAT] * n_outputs
        is_sequence = [0 if id not in seq_outputs else 1 for id in range(n_outputs)]
        is_optional = [0 if id not in optional_outputs else 1 for id in range(n_outputs)]
        output_shapes_cast = cast(List[List[int]],
                [[0] if isinstance(shape, str) else shape for shape in output_shapes]
        )
        outputs: List[ValueInfoProto] = []
        for (name, ttype, shape, is_seq, is_opt) in \
                zip(output_names, output_types, output_shapes_cast, is_sequence, is_optional):
            if is_seq:
                outputs += [helper.make_tensor_sequence_value_info(name, ttype, shape)]
            elif is_opt:
                type_proto = helper.make_tensor_type_proto(ttype, shape)
                optional_type_proto = helper.make_optional_type_proto(type_proto)
                outputs += [helper.make_value_info(name, optional_type_proto)]
            else:
                outputs += [helper.make_tensor_value_info(name, ttype, shape)]

        node = helper.make_node(op, input_names, output_names, **attrs)
        graph = helper.make_graph([node], op, inputs, outputs, initializer)
        original = helper.make_model(
            graph,
            producer_name='test',
            opset_imports=[helper.make_opsetid('', from_opset)]
        )
        onnx.checker.check_model(original)
        shape_inference.infer_shapes(original, strict_mode=True)

        converted = version_converter.convert_version(original, latest_opset)
        onnx.checker.check_model(converted)
        shape_inference.infer_shapes(converted, strict_mode=True)

    def test_Abs(self) -> None:
        self._test_op_upgrade('Abs', 1, attrs={'consumed_inputs': [0]})

    def test_Acosh(self) -> None:
        self._test_op_upgrade('Acosh', 9)

    def test_Acos(self) -> None:
        self._test_op_upgrade('Acos', 7)

    def test_And(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('And', 7, [[2, 3], [2, 3]], [[2, 3]],
            [TensorProto.BOOL, TensorProto.BOOL], [TensorProto.BOOL]
        )

    def test_Asinh(self) -> None:
        self._test_op_upgrade('Asinh', 9)

    def test_Atanh(self) -> None:
        self._test_op_upgrade('Atanh', 9)

    def test_Add_1(self) -> None:
        self._test_op_upgrade('Add', 1,
            [[3, 4, 5], [3, 4, 5]],
            attrs={'consumed_inputs': [0]}
        )

    def test_Add_2(self) -> None:
        self._test_op_upgrade('Add', 1, [[3, 4, 5], [5]],
            attrs={'consumed_inputs': [0], 'broadcast': 1}
        )

    def test_Add_3(self) -> None:
        self._test_op_upgrade('Add', 1, [[3, 4, 5], [3]],
            attrs={'consumed_inputs': [0], 'broadcast': 1, 'axis': 0}
        )

    def test_ArgMax_1(self) -> None:
        self._test_op_upgrade('ArgMax', 7, [[2, 3, 4]], [[1, 3, 4]],
            output_types=[TensorProto.INT64]
        )

    def test_ArgMax_2(self) -> None:
        self._test_op_upgrade('ArgMax', 7, [[2, 3, 4]], [[2, 1, 4]],
            output_types=[TensorProto.INT64],
            attrs={'axis': 1}
        )

    def test_ArgMin_1(self) -> None:
        self._test_op_upgrade('ArgMin', 7, [[2, 3, 4]], [[1, 3, 4]],
            output_types=[TensorProto.INT64]
        )

    def test_ArgMin_2(self) -> None:
        self._test_op_upgrade('ArgMin', 7, [[2, 3, 4]], [[2, 1, 4]],
            output_types=[TensorProto.INT64],
            attrs={'axis': 1}
        )

    def test_Asin(self) -> None:
        self._test_op_upgrade('Asin', 7)

    def test_Atan(self) -> None:
        self._test_op_upgrade('Atan', 7)

    def test_AveragePool(self) -> None:
        self._test_op_upgrade('AveragePool', 1, [[1, 1, 5, 5]], [[1, 1, 4, 4]],
            attrs={'kernel_shape': [2, 2]}
        )

    def test_Bernoulli(self) -> None:
        self._test_op_upgrade('Bernoulli', 15)

    def test_BitShift(self) -> None:
        self._test_op_upgrade('BitShift', 11, [[2, 3], [2, 3]], [[2, 3]],
            [TensorProto.UINT8, TensorProto.UINT8], [TensorProto.UINT8],
            attrs={'direction': 'RIGHT'}
        )

    def test_BatchNormalization_1(self) -> None:
        self._test_op_upgrade('BatchNormalization', 1, [[1, 3], [3], [3], [3], [3]], [[1, 3]],
            attrs={'consumed_inputs': [1, 1], 'is_test': 1, 'spatial': 1}
        )

    def test_BatchNormalization_2(self) -> None:
        self._test_op_upgrade('BatchNormalization', 14,
            [[1, 3], [3], [3], [3], [3]], [[1, 3], [3], [3]],
            attrs={'training_mode': 1}
        )

    def test_Cast(self) -> None:
        # 5->6 adapter is missing
        self._test_op_upgrade('Cast', 6, [[2, 3]], [[2, 3]], [TensorProto.INT64], attrs={'to': 1})

    def test_Ceil(self) -> None:
        self._test_op_upgrade('Ceil', 1, attrs={'consumed_inputs': [0]})

    def test_Celu(self) -> None:
        self._test_op_upgrade('Celu', 12)

    def test_Clip_1(self) -> None:
        self._test_op_upgrade('Clip', 1, attrs={'consumed_inputs': [0]})

    def test_Clip_2(self) -> None:
        self._test_op_upgrade('Clip', 1, attrs={'consumed_inputs': [0], 'min': -1.4})

    def test_Clip_3(self) -> None:
        self._test_op_upgrade('Clip', 1, attrs={'consumed_inputs': [0], 'max': 2.6})

    def test_Clip_4(self) -> None:
        self._test_op_upgrade('Clip', 1, attrs={'consumed_inputs': [0], 'min': -1.4, 'max': 2.6})

    def test_Compress(self) -> None:
        self._test_op_upgrade('Compress', 9, [[6, 7], [3]], [[3]],
            [TensorProto.FLOAT, TensorProto.BOOL], [TensorProto.FLOAT]
        )

    def test_Concat(self) -> None:
        self._test_op_upgrade('Concat', 1, [[2, 3], [2, 4]], [[2, 7]])

    def test_constant(self) -> None:
        value = helper.make_tensor(
            'Value',
            TensorProto.FLOAT,
            dims=[3, 4, 5],
            vals=np.random.rand(3, 4, 5).astype(np.float32).tobytes(),
            raw=True
        )
        self._test_op_upgrade('Constant', 1, [], attrs={'value': value})

    def test_ConstantOfShape(self) -> None:
        self._test_op_upgrade('ConstantOfShape', 9, [[3]])

    def test_Conv_1(self) -> None:
        self._test_op_upgrade('Conv', 1, [[1, 3, 5, 5], [4, 3, 2, 2], [4]], [[1, 4, 4, 4]])

    def test_Conv_2(self) -> None:
        self._test_op_upgrade('Conv', 1, [[1, 3, 5, 5], [4, 3, 2, 2], [4]], [[1, 4, 4, 4]])

    def test_Conv_3(self) -> None:
        self._test_op_upgrade('Conv', 1, [[1, 3, 5, 5], [4, 1, 2, 2], [4]], [[1, 4, 3, 7]],
            attrs={'dilations': [1, 2], 'group': 3, 'pads': [0, 1, 2, 3], 'strides': [2, 1]})

    def test_Convinteger(self) -> None:
        self._test_op_upgrade('ConvInteger', 10, [[1, 3, 5, 5], [4, 3, 2, 2], [4]], [[1, 4, 4, 4]],
            [TensorProto.UINT8, TensorProto.UINT8, TensorProto.UINT8], [TensorProto.INT32]
        )

    def test_ConvTranspose(self) -> None:
        self._test_op_upgrade('ConvTranspose', 1, [[1, 1, 5, 5], [1, 1, 3, 3]], [[1, 1, 7, 7]])

    def test_Cosh(self) -> None:
        self._test_op_upgrade('Cosh', 9)

    def test_Cos(self) -> None:
        self._test_op_upgrade('Cos', 7)

    def test_Cumsum(self) -> None:
        self._test_op_upgrade('CumSum', 11, [[3, 4, 5], []], [[3, 4, 5]],
            [TensorProto.FLOAT, TensorProto.INT64]
        )

    def test_DepthToSpace(self) -> None:
        self._test_op_upgrade('DepthToSpace', 1, [[1, 8, 3, 3]], [[1, 2, 6, 6]],
            attrs={'blocksize': 2}
        )

    def test_DequantizeLinear(self) -> None:
        self._test_op_upgrade('DequantizeLinear', 10, [[2, 3], [], []], [[2, 3]],
            [TensorProto.INT8, TensorProto.FLOAT, TensorProto.INT8]
        )

    def test_Det_1(self) -> None:
        self._test_op_upgrade('Det', 11, [[3, 5, 5]], [[3]])

    def test_Det_2(self) -> None:
        self._test_op_upgrade('Det', 11, [[5, 5]], [[]])

    def test_DynamicQuantizeLinear(self) -> None:
        self._test_op_upgrade('DynamicQuantizeLinear', 11, [[3, 4, 5]], [[3, 4, 5], [], []],
            output_types=[TensorProto.UINT8, TensorProto.FLOAT, TensorProto.UINT8]
        )

    def test_Div(self) -> None:
        self._test_op_upgrade('Div', 1, [[3, 4, 5], [3, 1, 5]], attrs={'consumed_inputs': [0]})

    def test_Dropout(self) -> None:
        self._test_op_upgrade('Dropout', 1, attrs={'consumed_inputs': [0], 'is_test': 1})

    def test_Einsum_1(self) -> None:
        self._test_op_upgrade('Einsum', 12, [[3, 4, 5], [3, 5, 6]], [[3, 4, 6]],
            attrs={'equation': 'bij, bjk -> bik'}
        )

    def test_Einsum_2(self) -> None:
        self._test_op_upgrade('Einsum', 12, [[4, 5]], [[5, 4]],
            attrs={'equation': 'ij->ji'}
        )

    def test_Elu(self) -> None:
        self._test_op_upgrade('Elu', 1, attrs={'consumed_inputs': [0]})

    def test_Equal(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('Equal', 7, [[2, 3], [2, 3]], [[2, 3]], output_types=[TensorProto.BOOL])

    def test_Erf(self) -> None:
        self._test_op_upgrade('Erf', 9)

    def test_Exp(self) -> None:
        self._test_op_upgrade('Exp', 1, attrs={'consumed_inputs': [0]})

    def test_Expand(self) -> None:
        shape = helper.make_tensor(
            'b',
            TensorProto.INT64,
            dims=[4],
            vals=np.array([5, 2, 6, 4])
        )
        self._test_op_upgrade('Expand', 8, [[2, 1, 4], [4]], [[5, 2, 6, 4]],
            [TensorProto.FLOAT, TensorProto.INT64],
            initializer=[shape]
        )

    def test_EyeLike(self) -> None:
        self._test_op_upgrade('EyeLike', 9, [[4, 5]], [[4, 5]])

    def test_Flatten(self) -> None:
        self._test_op_upgrade('Flatten', 1, [[3, 4, 5]], [[3, 20]], attrs={'axis': 1})

    def test_Floor(self) -> None:
        self._test_op_upgrade('Floor', 1, attrs={'consumed_inputs': [0]})

    def test_Gather(self) -> None:
        self._test_op_upgrade('Gather', 1, [[3, 4, 5], [6, 7]], [[6, 7, 4, 5]],
            [TensorProto.FLOAT, TensorProto.INT64]
        )

    def test_GatherElements(self) -> None:
        self._test_op_upgrade('GatherElements', 11, [[3, 4, 5], [6, 7]], [[6, 7]],
            [TensorProto.FLOAT, TensorProto.INT64]
        )

    def test_GatherND(self) -> None:
        self._test_op_upgrade('GatherND', 11, [[1, 2, 3], [1, 2, 3]], [[1, 2]])

    def test_Gemm(self) -> None:
        self._test_op_upgrade('Gemm', 1, [[5, 4], [4, 3], [3]], [[5, 3]])

    def test_GlobalAveragePool(self) -> None:
        self._test_op_upgrade('GlobalAveragePool', 1, [[1, 3, 10, 10]], [[1, 3, 1, 1]])

    def test_GlobalMaxPool(self) -> None:
        self._test_op_upgrade('GlobalMaxPool', 1, [[1, 3, 10, 10]], [[1, 3, 1, 1]])

    def test_GlobalLpPool(self) -> None:
        # 1->2 adapter is missing
        self._test_op_upgrade('GlobalLpPool', 2, [[1, 3, 10, 10]], [[1, 3, 1, 1]])

    def test_Greater(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('Greater', 7, [[2, 3], [2, 3]], [[2, 3]],
            output_types=[TensorProto.BOOL]
        )

    def test_GreaterOrEqual(self) -> None:
        self._test_op_upgrade('GreaterOrEqual', 12, [[2, 3], [2, 3]], [[2, 3]],
            output_types=[TensorProto.BOOL]
        )

    def test_GridSample(self) -> None:
        self._test_op_upgrade('GridSample', 16, [[1, 1, 3, 3], [1, 3, 3, 2]], [[1, 1, 3, 3]],
            input_types=[TensorProto.FLOAT, TensorProto.FLOAT],
            output_types=[TensorProto.FLOAT],
            attrs={'mode': 'nearest', 'padding_mode': 'border', 'align_corners': 1},
        )

    def test_GRU_1(self) -> None:
        # 2->3, 6->7 adapters are missing
        self._test_op_upgrade('GRU', 7,
            [[5, 3, 4], [1, 18, 4], [1, 18, 4]], [[5, 1, 3, 6], [1, 3, 6]],
            attrs={'hidden_size': 6}
        )

    def test_GRU_2(self) -> None:
        # 2->3, 6->7 adapters are missing
        self._test_op_upgrade('GRU', 7,
            [[5, 3, 4], [2, 18, 4], [2, 18, 4]], [[5, 2, 3, 6], [2, 3, 6]],
            attrs={'hidden_size': 6, 'direction': 'bidirectional'}
        )

    def test_GRU_3(self) -> None:
        # 2->3, 6->7 adapters are missing
        self._test_op_upgrade('GRU', 7,
            [[5, 3, 4], [1, 18, 4], [1, 18, 4], [1, 24], [5], [1, 5, 6]],
            [[5, 1, 3, 6], [1, 3, 6]],
            [TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT],
            attrs={'hidden_size': 6}
        )

    def test_HardSigmoid(self) -> None:
        self._test_op_upgrade('HardSigmoid', 1, attrs={'consumed_inputs': [0]})

    def test_HardSwish(self) -> None:
        self._test_op_upgrade('HardSwish', 14)

    def test_Hardmax(self) -> None:
        self._test_op_upgrade('Hardmax', 1)

    def test_Identity(self) -> None:
        self._test_op_upgrade('Identity', 1)

    def test_If(self) -> None:
        sub_output = [helper.make_tensor_value_info('out', TensorProto.FLOAT, [3, 4, 5])]
        then_tensor = helper.make_tensor(
            'Value',
            TensorProto.FLOAT,
            dims=[3, 4, 5],
            vals=np.random.rand(3, 4, 5).astype(np.float32).tobytes(), raw=True
        )
        then_node = helper.make_node('Constant', [], ['out'], value=then_tensor)
        then_graph = helper.make_graph([then_node], 'then_graph', [], sub_output, [])
        else_tensor = helper.make_tensor(
            'Value',
            TensorProto.FLOAT,
            dims=[3, 4, 5],
            vals=np.random.rand(3, 4, 5).astype(np.float32).tobytes(), raw=True
        )
        else_node = helper.make_node('Constant', [], ['out'], value=else_tensor)
        else_graph = helper.make_graph([else_node], 'else_graph', [], sub_output, [])
        self._test_op_upgrade('If', 1, [[0]], [[3, 4, 5]], [TensorProto.BOOL],
            attrs={'then_branch': then_graph, 'else_branch': else_graph}
        )

    def test_InstanceNormalization(self) -> None:
        self._test_op_upgrade('InstanceNormalization', 1, [[1, 3], [3], [3]], [[1, 3]],
            attrs={'consumed_inputs': [0]}
        )

    def test_IsInf(self) -> None:
        self._test_op_upgrade('IsInf', 10, [[2, 3]], [[2, 3]], output_types=[TensorProto.BOOL])

    def test_IsNaN(self) -> None:
        self._test_op_upgrade('IsNaN', 9, [[2, 3]], [[2, 3]], output_types=[TensorProto.BOOL])

    def test_LeakyRelu(self) -> None:
        self._test_op_upgrade('LeakyRelu', 1, attrs={'consumed_inputs': [0]})

    def test_Less(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('Less', 7, [[2, 3], [2, 3]], [[2, 3]], output_types=[TensorProto.BOOL])

    def test_LessOrEqual(self) -> None:
        self._test_op_upgrade('LessOrEqual', 12, [[2, 3], [2, 3]], [[2, 3]],
            output_types=[TensorProto.BOOL]
        )

    def test_Log(self) -> None:
        self._test_op_upgrade('Log', 1, attrs={'consumed_inputs': [0]})

    def test_LogSoftmax(self) -> None:
        self._test_op_upgrade('LogSoftmax', 1)

    def test_Loop_1(self) -> None:
        iter_count = onnx.helper.make_tensor_value_info('iter_count', onnx.TensorProto.INT64, [])
        cond_in = onnx.helper.make_tensor_value_info('cond_in', onnx.TensorProto.BOOL, [])
        x_in = onnx.helper.make_tensor_value_info('x_in', onnx.TensorProto.FLOAT, [1])
        cond_out = onnx.helper.make_tensor_value_info('cond_out', onnx.TensorProto.BOOL, [])
        x_out = onnx.helper.make_tensor_value_info('x_out', onnx.TensorProto.FLOAT, [1])
        x_scan = onnx.helper.make_tensor_value_info('x_scan', onnx.TensorProto.FLOAT, [1])
        const = onnx.helper.make_node(
            'Constant',
            inputs=[],
            outputs=['one'],
            value=onnx.helper.make_tensor(
                name='value',
                data_type=onnx.TensorProto.FLOAT,
                dims=[1],
                vals=np.array([1]).astype(np.float32).astype(float),
            )
        )
        add = onnx.helper.make_node(
            'Add',
            inputs=['x_in', 'one'],
            outputs=['x_out']
        )
        id_1 = onnx.helper.make_node(
            'Identity',
            inputs=['x_out'],
            outputs=['x_scan']
        )
        id_2 = onnx.helper.make_node(
            'Identity',
            inputs=['cond_in'],
            outputs=['cond_out']
        )
        loop_body = onnx.helper.make_graph(
            [const, add, id_1, id_2],
            'loop_body',
            [iter_count, cond_in, x_in],
            [cond_out, x_out, x_scan]
        )
        self._test_op_upgrade('Loop', 1, [[], '', [1]], [[1], [5, 1]],
            [TensorProto.INT64, TensorProto.BOOL, TensorProto.FLOAT],
            attrs={'body': loop_body}
        )

    def test_Loop_2(self) -> None:
        iter_count = onnx.helper.make_tensor_value_info('iter_count', onnx.TensorProto.INT64, [])
        cond_in = onnx.helper.make_tensor_value_info('cond_in', onnx.TensorProto.BOOL, [])
        x_in = onnx.helper.make_tensor_value_info('x_in', onnx.TensorProto.FLOAT, [2, 1])
        cond_out = onnx.helper.make_tensor_value_info('cond_out', onnx.TensorProto.BOOL, [])
        x_out = onnx.helper.make_tensor_value_info('x_out', onnx.TensorProto.FLOAT, [2, 1])
        squeeze = onnx.helper.make_node(
            'Squeeze',
            inputs=['x_in'],
            outputs=['squeeze_out'],
            axes=[1]
        )
        unsqueeze = onnx.helper.make_node(
            'Unsqueeze',
            inputs=['squeeze_out'],
            outputs=['x_out'],
            axes=[1]
        )
        identity = onnx.helper.make_node(
            'Identity',
            inputs=['cond_in'],
            outputs=['cond_out']
        )
        loop_body = onnx.helper.make_graph(
            [squeeze, unsqueeze, identity],
            'loop_body',
            [iter_count, cond_in, x_in],
            [cond_out, x_out]
        )
        self._test_op_upgrade('Loop', 12, [[], '', [2, 1]], [[2, 1]],
            [TensorProto.INT64, TensorProto.BOOL, TensorProto.FLOAT],
            attrs={'body': loop_body}
        )

    def test_LpNormalization(self) -> None:
        self._test_op_upgrade('LpNormalization', 1)

    def test_LpPool(self) -> None:
        # 1->2 adapter is missing
        self._test_op_upgrade('LpPool', 2, [[1, 1, 5, 5]], [[1, 1, 4, 4]],
            attrs={'kernel_shape': [2, 2]}
        )

    def test_LRN_1(self) -> None:
        self._test_op_upgrade('LRN', 1, attrs={'size': 3})

    def test_LRN_2(self) -> None:
        self._test_op_upgrade('LRN', 1, [[2, 3, 4, 5]], [[2, 3, 4, 5]],
            attrs={'size': 3}
        )

    def test_LSTM_1(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('LSTM', 7,
            [[5, 3, 4], [1, 24, 4], [1, 24, 4]],
            [[5, 1, 3, 6], [1, 3, 6], [1, 3, 6]],
            attrs={'hidden_size': 6}
        )

    def test_LSTM_2(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('LSTM', 7,
            [[5, 3, 4], [2, 24, 4], [2, 24, 4]],
            [[5, 2, 3, 6], [2, 3, 6], [2, 3, 6]],
            attrs={'hidden_size': 6, 'direction': 'bidirectional'}
        )

    def test_LSTM_3(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('LSTM', 7,
            [[5, 3, 4], [1, 24, 4], [1, 24, 4], [1, 48], [5], [1, 5, 6], [1, 5, 6], [1, 18]],
            [[5, 1, 3, 6], [1, 3, 6], [1, 3, 6]],
            [TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.FLOAT],
            attrs={'hidden_size': 6}
        )

    def test_MatMul_1(self) -> None:
        self._test_op_upgrade('MatMul', 1, [[2, 3], [3, 4]], [[2, 4]])

    def test_MatMul_2(self) -> None:
        self._test_op_upgrade('MatMul', 1, [[5, 2, 3], [5, 3, 4]], [[5, 2, 4]])

    def test_MatMulInteger_1(self) -> None:
        self._test_op_upgrade('MatMulInteger', 10, [[2, 3], [3, 4]], [[2, 4]],
            [TensorProto.INT8, TensorProto.INT8], [TensorProto.INT32]
        )

    def test_MatMulInteger_2(self) -> None:
        self._test_op_upgrade('MatMulInteger', 10, [[2, 3], [3, 4], [], []], [[2, 4]],
            [TensorProto.INT8, TensorProto.INT8, TensorProto.INT8, TensorProto.INT8],
            [TensorProto.INT32]
        )

    def test_MatMulInteger_3(self) -> None:
        self._test_op_upgrade('MatMulInteger', 10, [[2, 3], [3, 4], [2], [4]], [[2, 4]],
            [TensorProto.INT8, TensorProto.INT8, TensorProto.INT8, TensorProto.INT8],
            [TensorProto.INT32]
        )

    def test_Max(self) -> None:
        self._test_op_upgrade('Max', 1, [[2, 3, 4], [2, 3, 4]], [[2, 3, 4]],
            attrs={'consumed_inputs': [0]}
        )

    def test_MaxPool_1(self) -> None:
        self._test_op_upgrade('MaxPool', 1, [[1, 1, 5, 5]], [[1, 1, 4, 4]],
            attrs={'kernel_shape': [2, 2]}
        )

    def test_MaxPool_2(self) -> None:
        self._test_op_upgrade('MaxPool', 8, [[1, 1, 5, 5]], [[1, 1, 4, 4], [1, 1, 4, 4]],
            output_types=[TensorProto.FLOAT, TensorProto.INT64],
            attrs={'kernel_shape': [2, 2]}
        )

    def test_MaxRoiPool(self) -> None:
        self._test_op_upgrade('MaxRoiPool', 1, [[2, 3, 20, 20], [4, 5]], [[4, 3, 3, 3]],
            attrs={'pooled_shape': [3, 3]}
        )

    def test_MaxUnpool(self) -> None:
        self._test_op_upgrade('MaxUnpool', 9, [[1, 1, 5, 5], [1, 1, 5, 5]], [[1, 1, 6, 6]],
            [TensorProto.FLOAT, TensorProto.INT64],
            attrs={'kernel_shape': [2, 2]}
        )

    def test_Mean(self) -> None:
        self._test_op_upgrade('Mean', 1, [[2, 3, 4], [2, 3, 4]], [[2, 3, 4]],
            attrs={'consumed_inputs': [0]}
        )

    def test_MeanVarianceNormalization(self) -> None:
        self._test_op_upgrade('MeanVarianceNormalization', 9, attrs={'axes': [1, 2]})

    def test_Min(self) -> None:
        self._test_op_upgrade('Min', 1, [[2, 3, 4], [2, 3, 4]], [[2, 3, 4]],
            attrs={'consumed_inputs': [0]}
        )

    def test_Mish(self) -> None:
        self._test_op_upgrade('Mish', 18)

    def test_Mod_1(self) -> None:
        self._test_op_upgrade('Mod', 10, [[2, 3], [2, 3]], [[2, 3]])

    def test_Mod_2(self) -> None:
        self._test_op_upgrade('Mod', 10, [[2, 3], [2, 3]], [[2, 3]], attrs={'fmod': 1})

    def test_Mul(self) -> None:
        self._test_op_upgrade('Mul', 1, [[2, 3, 4], [2, 1, 4]], [[2, 3, 4]],
            attrs={'consumed_inputs': [0]}
        )

    def test_Multinomial(self) -> None:
        self._test_op_upgrade('Multinomial', 7, [[3, 5]], [[3, 7]],
            output_types=[TensorProto.INT32],
            attrs={'sample_size': 7}
        )

    def test_Neg(self) -> None:
        self._test_op_upgrade('Neg', 1, attrs={'consumed_inputs': [0]})

    def test_NegativeLogLikelihoodLoss_1(self) -> None:
        self._test_op_upgrade('NegativeLogLikelihoodLoss', 12, [[3, 4, 5], [3, 5]], [[]],
            [TensorProto.FLOAT, TensorProto.INT64]
        )

    def test_NegativeLogLikelihoodLoss_2(self) -> None:
        self._test_op_upgrade('NegativeLogLikelihoodLoss', 12, [[3, 4, 5], [3, 5], [4]], [[]],
            [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT]
        )

    def test_NonMaxSuppression(self) -> None:
        self._test_op_upgrade('NonMaxSuppression', 10, [[2, 3, 4], [3, 5, 6]], [[2, 3]],
            output_types=[TensorProto.INT64]
        )

    def test_NonZero(self) -> None:
        self._test_op_upgrade('NonZero', 9, [[3, 3]], [[2, 4]], output_types=[TensorProto.INT64])

    def test_Not(self) -> None:
        self._test_op_upgrade('Not', 1, [[2, 3]], [[2, 3]], [TensorProto.BOOL], [TensorProto.BOOL])

    def test_OneHot(self) -> None:
        self._test_op_upgrade('OneHot', 9, [[3, 4, 5], [], [2]], [[3, 4, 5, 6]])

    def test_Or(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('Or', 7, [[2, 3], [2, 3]], [[2, 3]],
            [TensorProto.BOOL, TensorProto.BOOL], [TensorProto.BOOL]
        )

    def test_Pad(self) -> None:
        # 1->2 adapter is missing
        self._test_op_upgrade('Pad', 2, [[3, 4]], [[5, 8]],
            attrs={'pads': [1, 2, 1, 2], 'value': 1.5}
        )

    def test_Pow(self) -> None:
        self._test_op_upgrade('Pow', 1, [[2, 3, 4], [2, 3, 4]], [[2, 3, 4]])

    def test_PRelu(self) -> None:
        self._test_op_upgrade('PRelu', 1, [[2, 3, 4], [2, 3, 4]], [[2, 3, 4]],
            attrs={'consumed_inputs': [0]}
        )

    def test_QLinearConv(self) -> None:
        self._test_op_upgrade('QLinearConv', 10,
            [[1, 3, 5, 5], [], [], [4, 3, 2, 2], [], [], [], []], [[1, 4, 4, 4]]
        )

    def test_QLinearMatMul(self) -> None:
        self._test_op_upgrade('QLinearMatMul', 10, [[2, 3], [], [], [3, 4], [], [], [], []], [[2, 4]])

    def test_QuantizeLinear(self) -> None:
        self._test_op_upgrade('QuantizeLinear', 10, [[3, 4, 5], [], []], [[3, 4, 5]],
            [TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.UINT8], [TensorProto.UINT8]
        )

    def test_RandomNormal(self) -> None:
        self._test_op_upgrade('RandomNormal', 1, [], [[3, 4, 5]], attrs={'shape': [3, 4, 5]})

    def test_RandomNormalLike(self) -> None:
        like = helper.make_tensor(
            'a',
            TensorProto.FLOAT,
            dims=[3, 4, 5],
            vals=np.random.rand(3, 4, 5).astype(np.float32).tobytes(), raw=True
        )
        self._test_op_upgrade('RandomNormalLike', 1, [[3, 4, 5]], [[3, 4, 5]],
            initializer=[like]
        )

    def test_RandomUniform(self) -> None:
        self._test_op_upgrade('RandomUniform', 1, [], [[3, 4, 5]], attrs={'shape': [3, 4, 5]})

    def test_RandomUniformLike(self) -> None:
        like = helper.make_tensor(
            'a',
            TensorProto.FLOAT,
            dims=[3, 4, 5],
            vals=np.random.rand(3, 4, 5).astype(np.float32).tobytes(), raw=True
        )
        self._test_op_upgrade('RandomUniformLike', 1, [[3, 4, 5]], [[3, 4, 5]],
            initializer=[like]
        )

    def test_Range(self) -> None:
        start = helper.make_tensor('a', TensorProto.FLOAT, dims=[], vals=np.array([0]))
        end = helper.make_tensor('b', TensorProto.FLOAT, dims=[], vals=np.array([12]))
        step = helper.make_tensor('c', TensorProto.FLOAT, dims=[], vals=np.array([2]))
        self._test_op_upgrade('Range', 11, [[], [], []], [[6]],
            initializer=[start, end, step]
        )

    def test_Reciprocal(self) -> None:
        self._test_op_upgrade('Reciprocal', 1, attrs={'consumed_inputs': [0]})

    def test_ReduceL1(self) -> None:
        self._test_op_upgrade('ReduceL1', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceL2(self) -> None:
        self._test_op_upgrade('ReduceL2', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceLogSum(self) -> None:
        self._test_op_upgrade('ReduceLogSum', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceLogSumExp(self) -> None:
        self._test_op_upgrade('ReduceLogSumExp', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceMean(self) -> None:
        self._test_op_upgrade('ReduceMean', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceMax(self) -> None:
        self._test_op_upgrade('ReduceMax', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceMin(self) -> None:
        self._test_op_upgrade('ReduceMin', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceProd(self) -> None:
        self._test_op_upgrade('ReduceProd', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceSum(self) -> None:
        self._test_op_upgrade('ReduceSum', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_ReduceSumSquare(self) -> None:
        self._test_op_upgrade('ReduceSumSquare', 1, [[3, 4, 5]], [[1, 1, 1]])

    def test_Relu(self) -> None:
        self._test_op_upgrade('Relu', 1, attrs={'consumed_inputs': [0]})

    def test_Reshape(self) -> None:
        self._test_op_upgrade('Reshape', 1, [[3, 4, 5]], [[3, 10, 2]],
            attrs={'consumed_inputs': [0], 'shape': [3, 10, 2]}
        )

    def test_Resize(self) -> None:
        self._test_op_upgrade('Resize', 10, [[3, 4, 5], [3]], [[3, 8, 15]])

    def test_ReverseSequence(self) -> None:
        self._test_op_upgrade('ReverseSequence', 10, [[3, 4, 5], [4]], [[3, 4, 5]],
            [TensorProto.FLOAT, TensorProto.INT64]
        )

    def test_RNN_1(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('RNN', 7, [[5, 3, 4], [1, 6, 4], [1, 6, 4]], [[5, 1, 3, 6], [1, 3, 6]],
            attrs={'hidden_size': 6}
        )

    def test_RNN_2(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('RNN', 7, [[5, 3, 4], [2, 6, 4], [2, 6, 4]], [[5, 2, 3, 6], [2, 3, 6]],
            attrs={'hidden_size': 6, 'direction': 'bidirectional'}
        )

    def test_RNN_3(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('RNN', 7,
            [[5, 3, 4], [1, 6, 4], [1, 6, 4], [1, 12], [5], [1, 5, 6]],
            [[5, 1, 3, 6], [1, 3, 6]],
            [TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT],
            attrs={'hidden_size': 6}
        )

    def test_RoiAlign_1(self) -> None:
        self._test_op_upgrade('RoiAlign', 10, [[2, 3, 20, 20], [10, 4], [10]], [[10, 3, 1, 1]],
            [TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.INT64]
        )

    def test_RoiAlign_2(self) -> None:
        self._test_op_upgrade('RoiAlign', 16, [[2, 3, 20, 20], [10, 4], [10]], [[10, 3, 1, 1]],
            [TensorProto.FLOAT, TensorProto.FLOAT, TensorProto.INT64],
            attrs={'coordinate_transformation_mode': 'half_pixel'}
        )

    def test_Round(self) -> None:
        self._test_op_upgrade('Round', 11)

    def test_Scatter(self) -> None:
        self._test_op_upgrade('Scatter', 9, [[2, 3], [1, 2], [1, 2]], [[2, 3]],
            [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT],
            [TensorProto.FLOAT]
        )

    def test_ScatterElements_1(self) -> None:
        self._test_op_upgrade('ScatterElements', 11, [[2, 3], [1, 2], [1, 2]], [[2, 3]],
            [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT],
            [TensorProto.FLOAT]
        )

    def test_ScatterElements_2(self) -> None:
        self._test_op_upgrade('ScatterElements', 16, [[2, 3], [1, 2], [1, 2]], [[2, 3]],
            [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT],
            [TensorProto.FLOAT],
            attrs={'reduction': 'add'}
        )

    def test_ScatterND_1(self) -> None:
        self._test_op_upgrade('ScatterND', 11, [[2, 3], [1, 2], [1, 2]], [[2, 3]],
            [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT],
            [TensorProto.FLOAT]
        )

    def test_ScatterND_2(self) -> None:
        self._test_op_upgrade('ScatterND', 16, [[2, 3], [1, 2], [1, 2]], [[2, 3]],
            [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT],
            [TensorProto.FLOAT],
            attrs={'reduction': 'mul'}
        )

    def test_Scan(self) -> None:
        sum_in = onnx.helper.make_tensor_value_info('sum_in', onnx.TensorProto.FLOAT, [2])
        next_in = onnx.helper.make_tensor_value_info('next_in', onnx.TensorProto.FLOAT, [2])
        sum_out = onnx.helper.make_tensor_value_info('sum_out', onnx.TensorProto.FLOAT, [2])
        scan_out = onnx.helper.make_tensor_value_info('scan_out', onnx.TensorProto.FLOAT, [2])
        add_node = onnx.helper.make_node(
            'Add',
            inputs=['sum_in', 'next_in'],
            outputs=['sum_out']
        )
        id_node = onnx.helper.make_node(
            'Identity',
            inputs=['sum_out'],
            outputs=['scan_out']
        )
        body = onnx.helper.make_graph(
            [add_node, id_node],
            'scan_body',
            [sum_in, next_in],
            [sum_out, scan_out]
        )
        self._test_op_upgrade('Scan', 8, ['', [1, 2], [1, 3, 2]], [[1, 2], [1, 3, 2]],
            attrs={'body': body, 'num_scan_inputs': 1}
        )

    def test_Selu(self) -> None:
        self._test_op_upgrade('Selu', 1, attrs={'consumed_inputs': [0]})

    def test_Shape(self) -> None:
        self._test_op_upgrade('Shape', 1, [[3, 4, 5]], [[3]], output_types=[TensorProto.INT64])

    def test_Shrink(self) -> None:
        self._test_op_upgrade('Shrink', 9)

    def test_Sigmoid(self) -> None:
        self._test_op_upgrade('Sigmoid', 1, attrs={'consumed_inputs': [0]})

    def test_Sign(self) -> None:
        self._test_op_upgrade('Sign', 9)

    def test_Sinh(self) -> None:
        self._test_op_upgrade('Sinh', 9)

    def test_Sin(self) -> None:
        self._test_op_upgrade('Sin', 7)

    def test_Size(self) -> None:
        self._test_op_upgrade('Size', 1, [[3, 4, 5]], [[]], output_types=[TensorProto.INT64])

    def test_Slice(self) -> None:
        self._test_op_upgrade('Slice', 1, [[3, 4, 5]], [[3, 2, 2]],
            attrs={'axes': [1, 2], 'starts': [0, 1], 'ends': [2, 3]}
        )

    def test_Softmax_0(self) -> None:
        self._test_op_upgrade('Softmax', 1, attrs={'axis': 0})

    def test_Softmax_1(self) -> None:
        self._test_op_upgrade('Softmax', 1, attrs={'axis': 1})

    def test_Softmax_2(self) -> None:
        self._test_op_upgrade('Softmax', 1, attrs={'axis': 2})

    def test_Softmax_3(self) -> None:
        self._test_op_upgrade('Softmax', 1, attrs={'axis': -1})

    def test_Softmax_4(self) -> None:
        self._test_op_upgrade('Softmax', 1, attrs={'axis': -2})

    def test_Softmax_5(self) -> None:
        self._test_op_upgrade('Softmax', 1, attrs={'axis': -3})

    def test_Softplus(self) -> None:
        self._test_op_upgrade('Softplus', 1)

    def test_Softsign(self) -> None:
        self._test_op_upgrade('Softsign', 1)

    def test_SoftmaxCrossEntropyLoss(self) -> None:
        self._test_op_upgrade('SoftmaxCrossEntropyLoss', 12, [[3, 4, 5, 6], [3, 6]], [[]],
            [TensorProto.FLOAT, TensorProto.INT64]
        )

    def test_SpaceToDepth(self) -> None:
        self._test_op_upgrade('SpaceToDepth', 1, [[1, 3, 8, 8]], [[1, 12, 4, 4]],
            attrs={'blocksize': 2}
        )

    def test_Split(self) -> None:
        # 1->2 adapter is missing
        self._test_op_upgrade('Split', 2, [[3, 4, 7]], [[3, 4, 2], [3, 4, 1], [3, 4, 4]],
            attrs={'axis': 2, 'split': [2, 1, 4]}
        )

    def test_Sqrt(self) -> None:
        self._test_op_upgrade('Sqrt', 1, attrs={'consumed_inputs': [0]})

    def test_Squeeze(self) -> None:
        self._test_op_upgrade('Squeeze', 1, [[2, 1, 3, 4, 1]], [[2, 3, 4]])

    def test_StringNormalizer(self) -> None:
        self._test_op_upgrade('StringNormalizer', 10, [[1, 3]], [[1, 3]],
            [TensorProto.STRING], [TensorProto.STRING],
            attrs={'case_change_action': 'LOWER'}
        )

    def test_Sub(self) -> None:
        self._test_op_upgrade('Sub', 1, [[2, 3, 4], [2, 3, 4]], [[2, 3, 4]],
            attrs={'consumed_inputs': [0]}
        )

    def test_Sum(self) -> None:
        self._test_op_upgrade('Sum', 1, [[2, 3, 4], [2, 3, 4]], [[2, 3, 4]],
            attrs={'consumed_inputs': [0]}
        )

    def test_Tanh(self) -> None:
        self._test_op_upgrade('Tanh', 1, attrs={'consumed_inputs': [0]})

    def test_Tan(self) -> None:
        self._test_op_upgrade('Tan', 7)

    def test_TfIdfVectorizer(self) -> None:
        self._test_op_upgrade('TfIdfVectorizer', 9, [[3]], [[5]],
            attrs={'max_gram_length': 3, 'max_skip_count': 1, 'min_gram_length': 2, 'mode': 'TFIDF', 'ngram_counts': [0, 20], 'ngram_indexes': [3, 4]}
        )

    def test_ThresholdedRelu(self) -> None:
        self._test_op_upgrade('ThresholdedRelu', 10)

    def test_Tile(self) -> None:
        # 5->6 adapter is missing
        repeats = helper.make_tensor('b', TensorProto.INT64, dims=[3], vals=np.array([1, 2, 3]))
        self._test_op_upgrade('Tile', 6, [[3, 4, 5], [3]], [[3, 8, 15]],
            [TensorProto.FLOAT, TensorProto.INT64],
            initializer=[repeats]
        )

    def test_TopK(self) -> None:
        self._test_op_upgrade('TopK', 1, [[3, 4, 5]], [[3, 4, 2], [3, 4, 2]],
            output_types=[TensorProto.FLOAT, TensorProto.INT64],
            attrs={'k': 2}
        )

    def test_Transpose(self) -> None:
        self._test_op_upgrade('Transpose', 1, [[1, 2, 5, 3, 7]], [[1, 7, 5, 2, 3]],
            attrs={'perm': [0, 4, 2, 1, 3]}
        )

    def test_Trilu(self) -> None:
        self._test_op_upgrade('Trilu', 14)

    def test_Unique_1(self) -> None:
        self._test_op_upgrade('Unique', 11, [[3, 4, 5]], [[None]])

    def test_Unique_2(self) -> None:
        self._test_op_upgrade('Unique', 11, [[3, 4, 5]], [[3, None, 5]],
            attrs={'axis': 1}
        )

    def test_Unsqueeze(self) -> None:
        self._test_op_upgrade('Unsqueeze', 1, [[3, 4, 5]], [[3, 4, 1, 5]], attrs={'axes': [2]})

    def test_Upsample(self) -> None:
        self._test_op_upgrade('Upsample', 1, [[1, 3, 4, 5]], [[1, 3, 6, 10]],
            attrs={'width_scale': 2., 'height_scale': 1.5}
        )

    def test_Where(self) -> None:
        self._test_op_upgrade('Where', 9, [[2, 3], [2, 3], [2, 3]], [[2, 3]],
            [TensorProto.BOOL, TensorProto.FLOAT, TensorProto.FLOAT]
        )

    def test_Xor(self) -> None:
        # 6->7 adapter is missing
        self._test_op_upgrade('Xor', 7, [[2, 3], [2, 3]], [[2, 3]],
            [TensorProto.BOOL, TensorProto.BOOL], [TensorProto.BOOL]
        )

    def test_CastLike(self) -> None:
        self._test_op_upgrade('CastLike', 15,
            [[2, 3, 4], [2, 1, 4]],
            [[2, 3, 4]],
            input_types=[TensorProto.FLOAT, TensorProto.FLOAT16],
            output_types=[TensorProto.FLOAT16])

    def test_LayerNormalization(self) -> None:
        self._test_op_upgrade('LayerNormalization', 17,
                              [[2, 3, 4, 5], [4, 5], [4, 5]],
                              [[2, 3, 4, 5]],
                              input_types=[TensorProto.FLOAT,
                                           TensorProto.FLOAT, TensorProto.FLOAT],
                              output_types=[TensorProto.FLOAT],
                              attrs={'axis': 2})

    def _test_window_function(self, window_function_name: str) -> None:
        size = helper.make_tensor('a', TensorProto.INT64, dims=[], vals=np.array([10]))
        self._test_op_upgrade(window_function_name,
                              17,
                              [[]],
                              [[10]],
                              [TensorProto.INT64],
                              initializer=[size])

    def test_BlackmanWindow(self) -> None:
        self._test_window_function('BlackmanWindow')

    def test_HannWindow(self) -> None:
        self._test_window_function('HannWindow')

    def test_HammingWindow(self) -> None:
        self._test_window_function('HammingWindow')

    def test_DFT(self) -> None:
        self._test_op_upgrade('DFT', 17, [[2, 16, 1], []], [[2, 16, 2]])
        self._test_op_upgrade('DFT', 17, [[2, 16, 2], []], [[2, 16, 2]])
        self._test_op_upgrade('DFT', 17, [[2, 16, 1], []], [[2, 9, 2]], attrs={'onesided': 1})
        self._test_op_upgrade('DFT', 17, [[2, 16, 2], []], [[2, 9, 2]], attrs={'onesided': 1})
        self._test_op_upgrade('DFT', 17, [[2, 16, 1], []], [[2, 16, 2]], attrs={'inverse': 1})
        self._test_op_upgrade('DFT', 17, [[2, 16, 2], []], [[2, 16, 2]], attrs={'inverse': 1})

    def _test_short_time_fourier_transform(self, operator_name: str) -> None:
        # Real
        signal = helper.make_tensor('a', TensorProto.FLOAT, dims=[2, 64], vals=np.random.rand(2, 64).astype(np.float32))
        frame_step = helper.make_tensor('b', TensorProto.INT64, dims=[1], vals=np.array([8]))
        window = helper.make_tensor('c', TensorProto.FLOAT, dims=[16], vals=np.ones(16).astype(np.float32))
        self._test_op_upgrade(operator_name,
                              17,
                              [[2, 64], [1], [16]],
                              [[2, 7, 16, 2]],
                              [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT, TensorProto.INT64],
                              initializer=[signal, frame_step, window])

        # Real Onesided
        signal = helper.make_tensor('a', TensorProto.FLOAT, dims=[2, 64], vals=np.random.rand(2, 64).astype(np.float32))
        frame_step = helper.make_tensor('b', TensorProto.INT64, dims=[1], vals=np.array([8]))
        window = helper.make_tensor('c', TensorProto.FLOAT, dims=[16], vals=np.ones(16).astype(np.float32))
        self._test_op_upgrade(operator_name,
                              17,
                              [[2, 64], [1], [16]],
                              [[2, 7, 9, 2]],
                              [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT, TensorProto.INT64],
                              attrs={'onesided': 1},
                              initializer=[signal, frame_step, window])

        # Complex
        signal = helper.make_tensor('a', TensorProto.FLOAT, dims=[2, 64, 2], vals=np.random.rand(2, 64, 2).astype(np.float32))
        frame_step = helper.make_tensor('b', TensorProto.INT64, dims=[1], vals=np.array([8]))
        window = helper.make_tensor('c', TensorProto.FLOAT, dims=[16], vals=np.ones(16).astype(np.float32))
        self._test_op_upgrade(operator_name,
                              17,
                              [[2, 64, 2], [1], [16]],
                              [[2, 7, 16, 2]],
                              [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT, TensorProto.INT64],
                              initializer=[signal, frame_step, window])

        # Complex Onesided
        signal = helper.make_tensor('a', TensorProto.FLOAT, dims=[2, 64, 2], vals=np.random.rand(2, 64, 2).astype(np.float32))
        frame_step = helper.make_tensor('b', TensorProto.INT64, dims=[1], vals=np.array([8]))
        window = helper.make_tensor('c', TensorProto.FLOAT, dims=[16], vals=np.ones(16).astype(np.float32))
        frame_length = helper.make_tensor('e', TensorProto.INT64, dims=[1], vals=np.array([16]))
        self._test_op_upgrade(operator_name,
                              17,
                              [[2, 64, 2], [1], [16]],
                              [[2, 7, 9, 2]],
                              [TensorProto.FLOAT, TensorProto.INT64, TensorProto.FLOAT, TensorProto.INT64],
                              attrs={'onesided': 1},
                              initializer=[signal, frame_step, window, frame_length])

    def test_STFT(self) -> None:
        self._test_short_time_fourier_transform('STFT')

    def test_MelWeightMatrix(self) -> None:
        num_mel_bins = helper.make_tensor('a', TensorProto.INT64, dims=[], vals=np.array([10]))
        dft_length = helper.make_tensor('b', TensorProto.INT64, dims=[], vals=np.array([64]))
        sample_rate = helper.make_tensor('c', TensorProto.INT64, dims=[], vals=np.array([0]))
        lower_edge_hertz = helper.make_tensor('d', TensorProto.FLOAT, dims=[], vals=np.array([0]))
        upper_edge_hertz = helper.make_tensor('e', TensorProto.FLOAT, dims=[], vals=np.array([1]))

        self._test_op_upgrade('MelWeightMatrix',
                              17,
                              [[], [], [], [], []],
                              [[33, 10]],
                              [TensorProto.INT64, TensorProto.INT64, TensorProto.INT64, TensorProto.FLOAT, TensorProto.FLOAT],
                              initializer=[num_mel_bins, dft_length, sample_rate, lower_edge_hertz, upper_edge_hertz])

        num_mel_bins = helper.make_tensor('a', TensorProto.INT64, dims=[], vals=np.array([20]))
        dft_length = helper.make_tensor('b', TensorProto.INT64, dims=[], vals=np.array([31]))
        sample_rate = helper.make_tensor('c', TensorProto.INT64, dims=[], vals=np.array([0]))
        lower_edge_hertz = helper.make_tensor('d', TensorProto.FLOAT, dims=[], vals=np.array([0]))
        upper_edge_hertz = helper.make_tensor('e', TensorProto.FLOAT, dims=[], vals=np.array([1]))

        self._test_op_upgrade('MelWeightMatrix',
                              17,
                              [[], [], [], [], []],
                              [[16, 20]],
                              [TensorProto.INT64, TensorProto.INT64, TensorProto.INT64, TensorProto.FLOAT, TensorProto.FLOAT],
                              initializer=[num_mel_bins, dft_length, sample_rate, lower_edge_hertz, upper_edge_hertz])

    def test_CenterCropPad(self) -> None:
        input = helper.make_tensor('input', TensorProto.FLOAT, dims=[2, 4], vals=np.array([1, 2, 3, 4, 5, 6, 7, 8]))
        shape = helper.make_tensor('shape', TensorProto.INT64, dims=[2], vals=np.array([3, 3]))
        self._test_op_upgrade('CenterCropPad',
                              18,
                              [[], []],
                              [[3, 3]],
                              [TensorProto.FLOAT, TensorProto.INT64],
                              initializer=[input, shape])

    def test_ops_tested(self) -> None:
        all_schemas = onnx.defs.get_all_schemas()
        all_op_names = [schema.name for schema in all_schemas if schema.domain == '']
        excluded_ops = [
            # Sequence-based and Optional-based ops disabled because
            # the version converter doesn't play nicely with sequences
            'ConcatFromSequence',
            'SequenceAt',
            'SequenceConstruct',
            'SequenceEmpty',
            'SequenceErase',
            'SequenceInsert',
            'SequenceLength',
            'SequenceMap',
            'SplitToSequence',
            'Optional',
            'OptionalGetElement',
            'OptionalHasElement',
        ]
        all_op_names = [op for op in all_op_names if op not in excluded_ops]

        untested_ops = set(all_op_names) - set(tested_ops)
        print(untested_ops)
        assert len(untested_ops) == 0


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# SPDX-License-Identifier: Apache-2.0

# Taken from https://github.com/dropbox/mypy-protobuf/blob/d984389124eae6dbbb517f766b9266bb32171510/python/protoc-gen-mypy
# (Apache 2.0 License)
# with own fixes to
# - appease flake8
# - exit without error when protobuf isn't installed
# - fix recognition of whether an identifier is defined locally
#   (unfortunately, we use a python package name ONNX_NAMESPACE_FOO_BAR_FOR_CI
#    on CI, which by the original protoc-gen-mypy script was recognized to be
#    camel case and therefore handled as an entry in the local package)


"""Protoc Plugin to generate mypy stubs. Loosely based on @zbarsky's go implementation"""

import sys
from collections import defaultdict
from contextlib import contextmanager
from typing import (
    Any,
    Callable,
    Dict,
    Generator,
    List,
    Set,
    cast,
    Optional
)

try:
    import google.protobuf.descriptor_pb2 as d_typed
    from google.protobuf.compiler import plugin_pb2 as plugin
except ImportError as e:
    sys.stderr.write(f'Failed to generate mypy stubs: {e}\n')
    sys.exit(0)


# Hax to get around fact that google protobuf libraries aren't in typeshed yet
d: Any = d_typed

GENERATED = "@ge" + "nerated"  # So phabricator doesn't think this file is generated
HEADER = f"# {GENERATED} by generate_proto_mypy_stubs.py.  Do not edit!\n"


class Descriptors:

    def __init__(self, request: plugin.CodeGeneratorRequest) -> None:
        files = {f.name: f for f in request.proto_file}
        to_generate = {n: files[n] for n in request.file_to_generate}
        self.files: Dict[str, d.FileDescriptorProto] = files
        self.to_generate: Dict[str, d.FileDescriptorProto] = to_generate
        self.messages: Dict[str, d.DescriptorProto] = {}
        self.message_to_fd: Dict[str, d.FileDescriptorProto] = {}

        def _add_enums(enums: d.EnumDescriptorProto, prefix: str, fd: d.FileDescriptorProto) -> None:
            for enum in enums:
                self.message_to_fd[prefix + enum.name] = fd

        def _add_messages(messages: d.DescriptorProto, prefix: str, fd: d.FileDescriptorProto) -> None:
            for message in messages:
                self.messages[prefix + message.name] = message
                self.message_to_fd[prefix + message.name] = fd
                sub_prefix = prefix + message.name + "."
                _add_messages(message.nested_type, sub_prefix, fd)
                _add_enums(message.enum_type, sub_prefix, fd)

        for fd in request.proto_file:
            start_prefix = "." + fd.package + "."
            _add_messages(fd.message_type, start_prefix, fd)
            _add_enums(fd.enum_type, start_prefix, fd)


class PkgWriter:
    """Writes a single pyi file"""

    def __init__(self, fd: d.FileDescriptorProto, descriptors: Descriptors) -> None:
        self.fd = fd
        self.descriptors = descriptors
        self.lines: List[str] = []
        self.indent = ""

        # dictionary of x->y for `from {x} import {y}`
        self.imports: Dict[str, Set[str]] = defaultdict(set)
        self.locals: Set[str] = set()

    def _import(self, path: str, name: str, import_as: Optional[str] = None) -> str:
        """Imports a stdlib path and returns a handle to it
        eg. self._import("typing", "Optional") -> "Optional"
        """
        imp = path.replace('/', '.')
        if import_as is not None:
            self.imports[imp].add(f"{name} as {import_as}")
            return import_as
        else:
            self.imports[imp].add(name)
            return name

    def _import_message(self, type_name: d.FieldDescriptorProto) -> str:
        """Import a referenced message and return a handle"""
        name = cast(str, type_name)

        if name[0] == '.' and name[1].isupper() and name[2].islower():
            # Message defined in this file
            return name[1:]

        message_fd = self.descriptors.message_to_fd[name]
        if message_fd.name == self.fd.name:
            # message defined in this package
            split = name.split('.')
            for i, segment in enumerate(split):
                if segment and segment[0].isupper() and segment[1].islower():
                    return ".".join(split[i:])

        # Not in package. Must import
        split = name.split(".")
        for i, segment in enumerate(split):
            if segment and segment[0].isupper() and segment[1].islower():
                assert message_fd.name.endswith('.proto')
                import_name = self._import(message_fd.name[:-6].replace('-', '_') + "_pb2", segment)
                remains = ".".join(split[i + 1:])
                if not remains:
                    return import_name
                raise AssertionError("Don't support nested imports yet")
                # return new_nested_import(import_name, remains)

        raise AssertionError("Could not parse local name " + name)

    @contextmanager  # type: ignore
    def _indent(self) -> Generator[None, None, None]:
        self.indent = self.indent + "    "
        yield
        self.indent = self.indent[:-4]

    def _write_line(self, line: str, *args: str) -> None:
        self.lines.append(self.indent + line.format(*args))

    def write_enums(self, enums: List[d.EnumDescriptorProto]) -> None:
        line = self._write_line
        for enum in enums:
            line("class {}(int):", enum.name)
            with self._indent():
                line("@classmethod")
                line("def Name(cls, number: int) -> str: ...")
                line("@classmethod")
                line("def Value(cls, name: str) -> int: ...")
                line("@classmethod")
                line("def keys(cls) -> {}[str]: ...",
                    self._import("typing", "List"))
                line("@classmethod")
                line("def values(cls) -> {}[int]: ...",
                    self._import("typing", "List"))
                line("@classmethod")
                line("def items(cls) -> {}[{}[str, int]]: ...",
                    self._import("typing", "List"),
                    self._import("typing", "Tuple"))

            for val in enum.value:
                line("{} = {}({}, {})", val.name, self._import("typing", "cast"), enum.name, val.number)
            line("")

    def write_messages(self, messages: List[d.DescriptorProto], prefix: str) -> None:
        line = self._write_line
        message_class = self._import("google.protobuf.message", "Message")

        for desc in messages:
            self.locals.add(desc.name)
            qualified_name = prefix + desc.name
            line("class {}({}):", desc.name, message_class)
            with self._indent():
                # Nested enums/messages
                self.write_enums(desc.enum_type)
                self.write_messages(desc.nested_type, qualified_name + ".")

                # Scalar fields
                for field in [f for f in desc.field if is_scalar(f)]:
                    if field.label == d.FieldDescriptorProto.LABEL_REPEATED:
                        container = self._import("google.protobuf.internal.containers", "RepeatedScalarFieldContainer")
                        line("{} = ... # type: {}[{}]", field.name, container, self.python_type(field))
                    else:
                        line("{} = ... # type: {}", field.name, self.python_type(field))
                line("")

                # Getters for non-scalar fields
                for field in [f for f in desc.field if not is_scalar(f)]:
                    line("@property")
                    if field.label == d.FieldDescriptorProto.LABEL_REPEATED:
                        msg = self.descriptors.messages[field.type_name]
                        if msg.options.map_entry:
                            # map generates a special Entry wrapper message
                            container = self._import("typing", "MutableMapping")
                            line("def {}(self) -> {}[{}, {}]: ...", field.name, container, self.python_type(msg.field[0]), self.python_type(msg.field[1]))
                        else:
                            container = self._import("google.protobuf.internal.containers", "RepeatedCompositeFieldContainer")
                            line("def {}(self) -> {}[{}]: ...", field.name, container, self.python_type(field))
                    else:
                        line("def {}(self) -> {}: ...", field.name, self.python_type(field))
                    line("")

                # Constructor
                line("def __init__(self,")
                with self._indent():
                    # Required args
                    for field in [f for f in desc.field if f.label == d.FieldDescriptorProto.LABEL_REQUIRED]:
                        line("{} : {},", field.name, self.python_type(field))
                    for field in [f for f in desc.field if f.label != d.FieldDescriptorProto.LABEL_REQUIRED]:
                        if field.label == d.FieldDescriptorProto.LABEL_REPEATED:
                            if field.type_name != '' and self.descriptors.messages[field.type_name].options.map_entry:
                                msg = self.descriptors.messages[field.type_name]
                                line("{} : {}[{}[{}, {}]] = None,", field.name, self._import("typing", "Optional", "OptionalType"),
                                    self._import("typing", "Mapping"), self.python_type(msg.field[0]), self.python_type(msg.field[1]))
                            else:
                                line("{} : {}[{}[{}]] = None,", field.name, self._import("typing", "Optional", "OptionalType"),
                                  self._import("typing", "Iterable"), self.python_type(field))
                        else:
                            line("{} : {}[{}] = None,", field.name, self._import("typing", "Optional", "OptionalType"),
                              self.python_type(field))
                    line(") -> None: ...")

                # Standard message methods
                line("@classmethod")
                line("def FromString(cls, s: bytes) -> {}: ...", qualified_name)
                line("def MergeFrom(self, other_msg: {}) -> None: ...", message_class)
                line("def CopyFrom(self, other_msg: {}) -> None: ...", message_class)
            line("")

    def write_services(self, services: d.ServiceDescriptorProto) -> None:
        line = self._write_line

        for service in services:
            # The service definition interface
            line("class {}({}, metaclass={}):", service.name, self._import("google.protobuf.service", "Service"), self._import("abc", "ABCMeta"))
            with self._indent():
                for method in service.method:
                    line("@{}", self._import("abc", "abstractmethod"))
                    line("def {}(self,", method.name)
                    with self._indent():
                        line("rpc_controller: {},", self._import("google.protobuf.service", "RpcController"))
                        line("request: {},", self._import_message(method.input_type))
                        line("done: {}[{}[[{}], None]],",
                          self._import("typing", "Optional"),
                          self._import("typing", "Callable"),
                          self._import_message(method.output_type))
                    line(") -> {}[{}]: ...", self._import("concurrent.futures", "Future"), self._import_message(method.output_type))

            # The stub client
            line("class {}({}):", service.name + "_Stub", service.name)
            with self._indent():
                line("def __init__(self, rpc_channel: {}) -> None: ...",
                  self._import("google.protobuf.service", "RpcChannel"))

    def python_type(self, field: d.FieldDescriptorProto) -> str:
        mapping: Dict[int, Callable[[], str]] = {
            d.FieldDescriptorProto.TYPE_DOUBLE: lambda: "float",
            d.FieldDescriptorProto.TYPE_FLOAT: lambda: "float",

            d.FieldDescriptorProto.TYPE_INT64: lambda: "int",
            d.FieldDescriptorProto.TYPE_UINT64: lambda: "int",
            d.FieldDescriptorProto.TYPE_FIXED64: lambda: "int",
            d.FieldDescriptorProto.TYPE_SFIXED64: lambda: "int",
            d.FieldDescriptorProto.TYPE_SINT64: lambda: "int",
            d.FieldDescriptorProto.TYPE_INT32: lambda: "int",
            d.FieldDescriptorProto.TYPE_UINT32: lambda: "int",
            d.FieldDescriptorProto.TYPE_FIXED32: lambda: "int",
            d.FieldDescriptorProto.TYPE_SFIXED32: lambda: "int",
            d.FieldDescriptorProto.TYPE_SINT32: lambda: "int",

            d.FieldDescriptorProto.TYPE_BOOL: lambda: "bool",
            d.FieldDescriptorProto.TYPE_STRING: lambda: "str",
            d.FieldDescriptorProto.TYPE_BYTES: lambda: "bytes",

            d.FieldDescriptorProto.TYPE_ENUM: lambda: self._import_message(field.type_name),
            d.FieldDescriptorProto.TYPE_MESSAGE: lambda: self._import_message(field.type_name),
            d.FieldDescriptorProto.TYPE_GROUP: lambda: self._import_message(field.type_name),
        }

        assert field.type in mapping, "Unrecognized type: " + field.type
        return mapping[field.type]()

    def write(self) -> str:
        imports = []
        for pkg, items in self.imports.items():
            imports.append(f"from {pkg} import (")
            for item in sorted(items):
                imports.append(f"    {item},")
            imports.append(")\n")

        return "\n".join(imports + self.lines)


def is_scalar(fd: d.FileDescriptorProto) -> bool:
    return not (
        fd.type == d.FieldDescriptorProto.TYPE_MESSAGE
        or fd.type == d.FieldDescriptorProto.TYPE_GROUP
    )


def generate_mypy_stubs(descriptors: Descriptors, response: plugin.CodeGeneratorResponse) -> None:
    for name, fd in descriptors.to_generate.items():
        pkg_writer = PkgWriter(fd, descriptors)
        pkg_writer.write_enums(fd.enum_type)
        pkg_writer.write_messages(fd.message_type, "")
        pkg_writer.write_services(fd.service)

        assert name == fd.name
        assert fd.name.endswith('.proto')
        output = response.file.add()
        output.name = fd.name[:-6].replace('-', '_') + '_pb2.pyi'
        output.content = HEADER + pkg_writer.write()
        print("Writing mypy to", output.name, file=sys.stderr)


def main() -> None:
    # Read request message from stdin
    data = sys.stdin.buffer.read()

    # Parse request
    request = plugin.CodeGeneratorRequest()
    request.ParseFromString(data)

    # Create response
    response = plugin.CodeGeneratorResponse()

    # Generate mypy
    generate_mypy_stubs(Descriptors(request), response)

    # Serialise response message
    output = response.SerializeToString()

    # Write to stdout
    sys.stdout.buffer.write(output)


if __name__ == '__main__':
    main()



# SPDX-License-Identifier: Apache-2.0

import importlib
import inspect
import sys
import pkgutil
from typing import Dict
from types import ModuleType


def collect_sample_implementations() -> Dict[str, str]:
    dict: Dict[str, str] = {}
    _recursive_scan(sys.modules[__name__], dict)
    return dict


def _recursive_scan(package: ModuleType, dict: Dict[str, str]) -> None:
    pkg_dir = package.__path__  # type: ignore
    module_location = package.__name__
    for _module_loader, name, ispkg in pkgutil.iter_modules(pkg_dir):  # type: ignore
        module_name = f"{module_location}.{name}"  # Module/package
        module = importlib.import_module(module_name)
        dict[name] = inspect.getsource(module)
        if ispkg:
            _recursive_scan(module, dict)



# SPDX-License-Identifier: Apache-2.0
import unittest

from onnx import defs, helper


class TestRelu(unittest.TestCase):

    def test_relu(self) -> None:
        self.assertTrue(defs.has('Relu'))
        helper.make_node(
            'Relu', ['X'], ['Y'])


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0

import itertools
import os
import platform
import unittest
import onnx.backend.base
import onnx.backend.test

from onnx.backend.base import Device, DeviceType
from onnx.backend.test.runner import BackendIsNotSupposedToImplementIt
import onnx.shape_inference
import onnx.version_converter
from typing import Optional, Any, Tuple, Sequence
from onnx import NodeProto, ModelProto, TensorProto
import numpy  # type: ignore

# The following just executes the fake backend through the backend test
# infrastructure. Since we don't have full reference implementation of all ops
# in ONNX repo, it's impossible to produce the proper results. However, we can
# run 'checker' (that's what base Backend class does) to verify that all tests
# fed are actually well-formed ONNX models.
#
# If everything is fine, all the tests would be marked as "skipped".
#
# We don't enable report in this test because the report collection logic itself
# fails when models are mal-formed.


class DummyBackend(onnx.backend.base.Backend):
    @classmethod
    def prepare(cls,
                model: ModelProto,
                device: str = 'CPU',
                **kwargs: Any
                ) -> Optional[onnx.backend.base.BackendRep]:
        super().prepare(model, device, **kwargs)

        # test strict shape inference
        onnx.checker.check_model(model)
        model = onnx.shape_inference.infer_shapes(model, check_type=True, strict_mode=True)

        value_infos = {vi.name: vi for vi in itertools.chain(model.graph.value_info, model.graph.output)}

        if do_enforce_test_coverage_safelist(model):
            for node in model.graph.node:
                for i, output in enumerate(node.output):
                    if node.op_type == 'Dropout' and i != 0:
                        continue
                    assert output in value_infos
                    tt = value_infos[output].type.tensor_type
                    assert tt.elem_type != TensorProto.UNDEFINED
                    for dim in tt.shape.dim:
                        assert dim.WhichOneof('value') == 'dim_value'

        raise BackendIsNotSupposedToImplementIt(
            "This is the dummy backend test that doesn't verify the results but does run the checker")

    @classmethod
    def run_node(cls,
                 node: NodeProto,
                 inputs: Any,
                 device: str = 'CPU',
                 outputs_info: Optional[Sequence[Tuple[numpy.dtype, Tuple[int, ...]]]] = None,
                 **kwargs: Any
                 ) -> Optional[Tuple[Any, ...]]:
        super().run_node(node, inputs, device=device, outputs_info=outputs_info)
        raise BackendIsNotSupposedToImplementIt(
            "This is the dummy backend test that doesn't verify the results but does run the checker")

    @classmethod
    def supports_device(cls, device: str) -> bool:
        d = Device(device)
        if d.type == DeviceType.CPU:
            return True
        return False


test_coverage_safelist = {
    'bvlc_alexnet', 'densenet121', 'inception_v1', 'inception_v2',
    'resnet50', 'shufflenet', 'SingleRelu', 'squeezenet_old', 'vgg19', 'zfnet'}


def do_enforce_test_coverage_safelist(model: ModelProto) -> bool:
    if model.graph.name not in test_coverage_safelist:
        return False
    for node in model.graph.node:
        if node.op_type in {'RNN', 'LSTM', 'GRU'}:
            return False
    return True


backend_test = onnx.backend.test.BackendTest(DummyBackend, __name__)
if os.getenv('APPVEYOR'):
    backend_test.exclude(r'(test_vgg19|test_zfnet)')
if platform.architecture()[0] == '32bit':
    backend_test.exclude(r'(test_vgg19|test_zfnet|test_bvlc_alexnet)')

# import all test cases at global scope to make them visible to python.unittest
globals().update(backend_test
                 .test_cases)

if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: Apache-2.0
# Originally there are some checker failures in weekly-CI
# Skip them in test_model_zoo.py for now
# TODO: fix these checker failures
SKIP_CHECKER_MODELS = {'vision/classification/alexnet/model/bvlcalexnet-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/caffenet/model/caffenet-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/densenet-121/model/densenet-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/inception_and_googlenet/inception_v1/model/inception-v1-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/inception_and_googlenet/inception_v2/model/inception-v2-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/rcnn_ilsvrc13/model/rcnn-ilsvrc13-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/resnet/model/resnet50-caffe2-v1-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/shufflenet/model/shufflenet-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/squeezenet/model/squeezenet1.0-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/vgg/model/vgg19-caffe2-3.onnx',  # opset1 typeinference function missing
                       'vision/classification/zfnet-512/model/zfnet512-3.onnx'}  # opset1 typeinference function missing



from typing import Dict, Union


class CheckerContext:
    ir_version: int = ...
    opset_imports: Dict[str, int] = ...


class ValidationError(Exception):
    ...


def check_value_info(bytes: bytes, checker_context: CheckerContext) -> None: ...
def check_tensor(bytes: bytes, checker_context: CheckerContext) -> None: ...
def check_sparse_tensor(bytes: bytes, checker_context: CheckerContext) -> None: ...
def check_attribute(bytes: bytes, checker_context: CheckerContext) -> None: ...
def check_node(bytes: bytes, checker_context: CheckerContext) -> None: ...
def check_graph(bytes: bytes, checker_context: CheckerContext) -> None: ...
def check_model(bytes: bytes) -> None: ...
def check_model_path(path: str) -> None: ...
